import QtQuick 2.12
import QtQuick.Window 2.12
import QtQuick.Controls 2.3

Item {
    signal uiButtonPressed(int role);
    signal quranButtonPressed();

    function hideBackgrounds()
    {
        city.background.visible     = false;
        region.background.visible   = false;
        country.background.visible  = false;
        timezone.background.visible = false;

        gregorian.background.visible = false;
        hijri.background.visible     = false;
        shamsi.background.visible    = false;

        westernArabic.background.visible = false;
        easternArabic.background.visible = false;
        farsi.background.visible         = false;

        prayer1.background.visible = false;
        prayer2.background.visible = false;
        prayer3.background.visible = false;
        prayer4.background.visible = false;
        prayer5.background.visible = false;
        prayer6.background.visible = false;
    }

    function showBackgrounds()
    {
        city.background.visible     = true
        region.background.visible   = true;
        country.background.visible  = true;
        timezone.background.visible = true;

        gregorian.background.visible = true;
        hijri.background.visible     = true;
        shamsi.background.visible    = true;

        westernArabic.background.visible = true;
        easternArabic.background.visible = true;
        farsi.background.visible         = true;

        prayer1.background.visible = true;
        prayer2.background.visible = true;
        prayer3.background.visible = true;
        prayer4.background.visible = true;
        prayer5.background.visible = true;
        prayer6.background.visible = true;
    }

    function setCurrentPrayer(prayerName)
    {
        currentPrayerName.text = prayerName

        prayers.visible       = false
        currentPrayer.visible = true
    }

    function setDate(gregorianDate, hijriDate, shamsiDate)
    {
        if (gregorianDate) {
            gregorian.visible = true;
            gregorian.text = gregorianDate;
        }
        else {
            gregorian.visible = false;
        }

        if (hijriDate) {
            hijri.visible = true;
            hijri.text = hijriDate;
        }
        else {
            hijri.visible = false;
        }

        if (shamsiDate) {
            shamsi.visible = true;
            shamsi.text = shamsiDate;
        }
        else {
            shamsi.visible = false;
        }
    }


    function setErrorStr(errorStr)
    {
        mainError.text = errorStr;
    }


    function setFont(latinFamily, arabicFamily, size, prayersArabic)
    {
        city.font.pointSize     = size;
        region.font.pointSize   = size * .8;
        country.font.pointSize  = size * .8;
        timezone.font.pointSize = size;

        gregorian.font.pointSize = size;
        hijri.font.pointSize     = size;
        shamsi.font.pointSize    = size;

        westernArabic.font.pointSize = size * 2.9;
        easternArabic.font.pointSize = size * 2.9;
        farsi.font.pointSize         = size * 2.9;

        prayer1.font.pointSize = size;
        prayer2.font.pointSize = size;
        prayer3.font.pointSize = size;
        prayer4.font.pointSize = size;
        prayer5.font.pointSize = size;
        prayer6.font.pointSize = size;

        currentPrayerName.font.pointSize = size * 2.9;
        uiButton.font.pointSize    = size * 1.33;

        mainError.font.pointSize = size * .8;
        mainError.font.family    = latinFamily;

        if (latinFamily) {
            city.font.family     = latinFamily;
            region.font.family   = latinFamily;
            country.font.family  = latinFamily;
            timezone.font.family = latinFamily;

            gregorian.font.family     = latinFamily;
            westernArabic.font.family = latinFamily;

            uiButton.font.family = latinFamily;

            if (!prayersArabic) {
                prayer1.font.family = latinFamily;
                prayer2.font.family = latinFamily;
                prayer3.font.family = latinFamily;
                prayer4.font.family = latinFamily;
                prayer5.font.family = latinFamily;
                prayer6.font.family = latinFamily;

                currentPrayerName.font.family = latinFamily;
            }
        }
        if (arabicFamily) {
            hijri.font.family         = arabicFamily;
            shamsi.font.family        = arabicFamily;
            easternArabic.font.family = arabicFamily;
            farsi.font.family         = arabicFamily;

            hijri.font.pointSize         = size * 1.2;
            shamsi.font.pointSize        = size * 1.2;
            easternArabic.font.pointSize = size * 3.1;
            farsi.font.pointSize         = size * 3.1;

            if (prayersArabic) {
                prayer1.font.family = arabicFamily;
                prayer2.font.family = arabicFamily;
                prayer3.font.family = arabicFamily;
                prayer4.font.family = arabicFamily;
                prayer5.font.family = arabicFamily;
                prayer6.font.family = arabicFamily;

                currentPrayerName.font.family = arabicFamily;

                prayer1.font.pointSize = size * 1.2;
                prayer2.font.pointSize = size * 1.2;
                prayer3.font.pointSize = size * 1.2;
                prayer4.font.pointSize = size * 1.2;
                prayer5.font.pointSize = size * 1.2;
                prayer6.font.pointSize = size * 1.2;

                currentPrayerName.font.pointSize = size * 3.1;
            }
        }
    }

    function setLocation(cityName, regionName, countryName, timezoneName)
    {
        city.text     = cityName;
        region.text   = regionName;
        country.text  = countryName;
        timezone.text = timezoneName;
    }

    function setPrayer(number, name, timeString, highlightPercent)
    {
        currentPrayer.visible = false
        prayers.visible       = true

        if (highlightPercent > 1) {
            highlightPercent = 1;
        }
        if (highlightPercent < 0) {
            highlightPercent = 0;
        }

        var show = name + "\n" + timeString;

        if (number === 1) {
            prayer1.text             = show;
            prayer1.highlightPercent = highlightPercent;
        }
        else if (number === 2) {
            prayer2.text             = show;
            prayer2.highlightPercent = highlightPercent;
        }
        else if (number === 3) {
            prayer3.text             = show;
            prayer3.highlightPercent = highlightPercent;
        }
        else if (number === 4) {
            prayer4.text             = show;
            prayer4.highlightPercent = highlightPercent;
        }
        else if (number === 5) {
            prayer5.text             = show;
            prayer5.highlightPercent = highlightPercent;
        }
        else if (number === 6) {
            prayer6.text             = show;
            prayer6.highlightPercent = highlightPercent;
        }
    }


    function setQuranButtonVisible(quranButtonVisible)
    {
        internal.quranButtonVisible = quranButtonVisible;
    }


    function setUIButtonRole(uiRole)
    {
        internal.uiButtonRole = uiRole;

        if (uiRole === 1) {
            uiButton.enabled       = true;
            uiButton.text          = "\u2699";
            uiButton.bottomPadding = 5;
            return;
        }

        if (uiRole === 2) {
            uiButton.enabled       = true;
            uiButton.text          = "\u25A0";
            uiButton.bottomPadding = 8;
            return;
        }

        uiButton.enabled       = false;
        uiButton.text          = "\u2699";
        uiButton.bottomPadding = 5;
    }


    function setTime(westernTime, easternTime, farsiTime)
    {
        if (westernTime) {
            westernArabic.visible = true;
            westernArabic.text = westernTime;
        }
        else {
            westernArabic.visible = false;
        }

        if (easternTime) {
            easternArabic.visible = true;
            easternArabic.text = easternTime;
        }
        else {
            easternArabic.visible = false;
        }

        if (farsiTime) {
            farsi.visible = true;
            farsi.text = farsiTime;
        }
        else {
            farsi.visible = false;
        }
    }


    QtObject {
        id: internal

        property int  uiButtonRole: 0
        property bool quranButtonVisible: false
    }


    Item {
        id: location
        anchors.top: parent.top
        anchors.left: parent.left
        anchors.right: parent.right
        anchors.topMargin: 7
        anchors.leftMargin: 7
        anchors.rightMargin: 7
        height: Math.max(city.height, timezone.height)

        LabelBkg {
            id: city
            anchors.top: parent.top
            anchors.left: parent.left
        }
        LabelBkg {
            id: region
            anchors.verticalCenter: city.verticalCenter
            anchors.left: city.right
            anchors.leftMargin: 7
        }
        LabelBkg {
            id: country
            anchors.verticalCenter: region.verticalCenter
            anchors.left: region.right
            anchors.leftMargin: 7
        }

        LabelBkg {
            id: timezone
            anchors.top: parent.top
            anchors.right: parent.right
        }
    }

    LabelBkg {
        id: gregorian
        anchors.top: location.bottom
        anchors.horizontalCenter: parent.horizontalCenter
        anchors.topMargin: 14
    }
    LabelBkg {
        id: hijri
        anchors.top: gregorian.bottom
        anchors.horizontalCenter: parent.horizontalCenter
        anchors.topMargin: 7
    }
    LabelBkg {
        id: shamsi
        anchors.top: hijri.bottom
        anchors.horizontalCenter: parent.horizontalCenter
        anchors.topMargin: 7
    }

    Row {
        id: clock
        anchors.verticalCenter: parent.verticalCenter
        anchors.horizontalCenter: parent.horizontalCenter
        anchors.topMargin: 14
        height: Math.max(westernArabic.height, easternArabic.height, farsi.height)
        spacing: 14

        LabelClock {
            id: westernArabic
            anchors.verticalCenter: parent.verticalCenter
        }
        LabelClock {
            id: easternArabic
            anchors.verticalCenter: parent.verticalCenter
        }
        LabelClock {
            id: farsi
            anchors.verticalCenter: parent.verticalCenter
        }
    }

    Row {
        id: prayers
        anchors.top: clock.bottom
        anchors.horizontalCenter: parent.horizontalCenter
        anchors.topMargin: 14
        spacing: 7

        LabelBkg {
            id: prayer1
            anchors.verticalCenter: parent.verticalCenter
            horizontalAlignment: Text.AlignHCenter
            verticalAlignment: Text.AlignVCenter
        }
        LabelBkg {
            id: prayer2
            anchors.verticalCenter: parent.verticalCenter
            horizontalAlignment: Text.AlignHCenter
            verticalAlignment: Text.AlignVCenter
        }
        LabelBkg {
            id: prayer3
            anchors.verticalCenter: parent.verticalCenter
            horizontalAlignment: Text.AlignHCenter
            verticalAlignment: Text.AlignVCenter
        }
        LabelBkg {
            id: prayer4
            anchors.verticalCenter: parent.verticalCenter
            horizontalAlignment: Text.AlignHCenter
            verticalAlignment: Text.AlignVCenter
        }
        LabelBkg {
            id: prayer5
            anchors.verticalCenter: parent.verticalCenter
            horizontalAlignment: Text.AlignHCenter
            verticalAlignment: Text.AlignVCenter
        }
        LabelBkg {
            id: prayer6
            anchors.verticalCenter: parent.verticalCenter
            horizontalAlignment: Text.AlignHCenter
            verticalAlignment: Text.AlignVCenter
        }
    }

    Row {

        id: currentPrayer
        anchors.top: clock.bottom
        anchors.horizontalCenter: parent.horizontalCenter
        anchors.topMargin: 14
        visible: false

        Label {
            id: currentPrayerName
            anchors.verticalCenter: parent.verticalCenter
            font.bold: true
            horizontalAlignment: Text.AlignHCenter
            verticalAlignment: Text.AlignVCenter
            padding: 5

            background: Rectangle {
                color: "#FFDBBA"
                radius: 11
                border.color: "#888888"
            }
        }
    }

    Label {
        id: mainError
        anchors.left: parent.left
        anchors.right: uiButton.left
        anchors.verticalCenter: uiButton.verticalCenter
        anchors.leftMargin: 7
        anchors.rightMargin: 7
        font.bold: true
        font.italic: true
        color: "#7A5544"
    }

    Button {
        id: uiButton
        anchors.right: parent.right
        anchors.bottom: parent.bottom
        anchors.rightMargin: 7
        anchors.bottomMargin: 7

        onClicked: uiButtonPressed(internal.uiButtonRole);

        background: Rectangle {
            color: uiButton.pressed || !uiButton.enabled ? Qt.rgba(1, 1, 1, .33) : Qt.rgba(1, 1, 1, .66)
            radius: 4
            border.color: "#000000"
        }
    }

    Button {
        id: quranButton
        anchors.right: parent.right
        anchors.bottom: uiButton.top
        anchors.rightMargin: 7
        anchors.bottomMargin: 7
        width: uiButton.width
        enabled: internal.quranButtonVisible && (internal.uiButtonRole === 1)
        visible: internal.quranButtonVisible
        text: "\u0642"                                 // book icon: \uD83D\uDCD7"

        onClicked: quranButtonPressed();

        background: Rectangle {
            color: quranButton.pressed || !quranButton.enabled ? Qt.rgba(1, 1, 1, .33) : Qt.rgba(1, 1, 1, .66)
            radius: 4
            border.color: "#000000"
        }
    }
}
