import QtQuick 2.12
import QtQuick.Window 2.12
import QtQuick.Controls 2.3

Item {
    signal ayahChanged(int surah, int ayah);
    signal closePressed();
    signal audioTogglePressed(int role, int surah, int ayah);


    function audioToggleRole(isStop)
    {
        if (isStop === undefined) {
            isStop = internal.buttonRole === 3;
        }

        if (isStop) {
            internal.buttonRole = 2;
            audioToggle.text    = "\u25A0";
        }
        else {
            internal.buttonRole = 3;
            audioToggle.text    = "\u25B6";
        }

        audioToggle.enabled = true;
    }


    function nextAyah(sendAudioToggle)
    {
        if (ayah.value < ayah.to) {
            ayah.value = ayah.value + 1;
        }
        else {
            ayah.value = ayah.from;
            if (surah.value < surah.to) {
                surah.value = surah.value + 1;
            }
            else {
                surah.value = surah.from;
            }
        }

        if (sendAudioToggle) {
            audioTogglePressed(3, surah.value, ayah.value);
        }
    }


    function setAyah(text, fontFamily)
    {
        ayahText.text        = text;
        ayahText.font.family = fontFamily ? fontFamily : internal.ayahFontFamily;
    }


    function setBusy(busy)
    {
        quranBusy.visible = busy;
    }


    function setErrorStr(errorStr)
    {
        quranError.text = errorStr;
    }


    function setFont(latinFamily, arabicFamily, size, settingsArabic)
    {
        var familyToUse = settingsArabic ? arabicFamily : latinFamily;
        var sizeToUse   = (settingsArabic && arabicFamily) ? size * 1.2 : size;

        surah.font.pointSize       = sizeToUse;
        ayah.font.pointSize        = sizeToUse;
        textMetrics.font.pointSize = sizeToUse;
        close.font.pointSize       = sizeToUse;

        quranError.font.pointSize = size * .8;
        quranError.font.family    = latinFamily;

        ayahText.font.family = arabicFamily;

        if (familyToUse) {
            surah.font.family       = familyToUse;
            //surah.popup.font.family = familyToUse;
            ayah.font.family        = familyToUse;
            textMetrics.font.family = familyToUse;
            close.font.family       = familyToUse;
        }

        textMetrics.text                = close.text;
        close.background.implicitHeight = textMetrics.height;

        internal.ayahFontFamily = ayahText.font.family;
    }


    function setSuwar(suwar, selected, ayahSelected)
    {
        suwarModel.clear();

        var textHeight = 0;
        var textWidth  = 0;

        for (var i = 0; i < suwar.length; i++) {
            var newSurah = {
                id: suwar[i].id,
                name: suwar[i].name,
                ayat: suwar[i].ayat
            };
            suwarModel.append(newSurah);

            textMetrics.text = newSurah.name;
            if (textMetrics.height > textHeight) {
                textHeight = textMetrics.height;
            }
            if (textMetrics.width > textWidth) {
                textWidth = textMetrics.width;
            }
        }

        surah.value = selected;
        ayah.to     = suwarModel.get(selected - 1).ayat;
        ayah.value  = ayahSelected;

        if (textHeight > 0) {
            surah.background.implicitHeight = textHeight;
            ayah.background.implicitHeight  = textHeight;
        }
        if (textWidth > 0) {
            surah.width = textWidth + surah.padding * 2 + surah.up.indicator.width + surah.down.indicator.width
        }
    }


    QtObject {
        id: internal

        property string ayahFontFamily;
        property int    buttonRole: 3;
    }

    ListModel {
        id: suwarModel
    }
    TextMetrics {
        id: textMetrics
    }
    Timer {
        id: ayahValueChangedTimer
        interval: 250

        onTriggered: {
            if ((suwarModel.count > 0) && (ayah.to > 1)) {
                ayahChanged(surah.value, ayah.value);
            }
        }
    }
    Timer {
        id: recalculateAyahTextPointSize
        interval: 10

        onTriggered: {
            if (ayahText.text.length > 0) {
                ayahText.font.pointSize = 99;
                while ((ayahText.font.pointSize >= 8) && (ayahText.height + 24 > ayahTextContainer.height)) {
                    ayahText.font.pointSize--;
                }
                ayahText.visible = true;
            }
        }
    }

    Item {
        id: selectors
        anchors.top: parent.top
        anchors.left: parent.left
        anchors.right: parent.right
        anchors.topMargin: 7
        anchors.leftMargin: 7
        anchors.rightMargin: 7
        height: surah.height //Math.max(surah.height, reciter.height)

        SpinBoxQuran {
            id: surah
            anchors.top: parent.top
            anchors.left: parent.left
            to: suwarModel.count

            textFromValue: function(value) {
                return suwarModel.get(value - 1).name;
            }

            valueFromText: function(text) {
                var textComp = text.toLowerCase;

                for (var i = 0; i < suwarModel.count; i++) {
                    if (suwarModel.get(i).name.toLowerCase() === textComp)
                        return i + 1;
                }
                return 0;
            }

            onValueChanged: {
                if (suwarModel.count > 0) {
                    ayah.value = 1;
                    ayah.to    = suwarModel.get(value - 1).ayat;
                    ayahValueChangedTimer.restart();
                }
            }
        }
        SpinBoxQuran {
            id: ayah
            anchors.top: parent.top
            anchors.left: surah.right
            anchors.leftMargin: 3
            implicitWidth: 95

            onValueChanged: {
                if (suwarModel.count > 0) {
                    ayahValueChangedTimer.restart();
                }
            }
        }

        Button {
            id: audioToggle
            anchors.top: parent.top
            anchors.right: parent.right
            text: "\u25B6"
            onClicked: {
                enabled: false;
                audioTogglePressed(internal.buttonRole, surah.value, ayah.value);
            }

            background: Rectangle {
                color: audioToggle.pressed || !audioToggle.enabled ? Qt.rgba(1, 1, 1, .33) : Qt.rgba(1, 1, 1, .66)
                radius: 4
                border.color: "#000000"
            }
        }
    }

    Rectangle {
        id: ayahTextContainer

        anchors.top: selectors.bottom
        anchors.bottom: close.top
        anchors.left: parent.left
        anchors.right: parent.right
        anchors.topMargin: 7
        anchors.bottomMargin: 7
        anchors.leftMargin: 7
        anchors.rightMargin: 7

        color: Qt.rgba(.75, .75, .75, .66)
        radius: 11
        border.color: "#FFDBBA"

        onWidthChanged: {
            ayahText.visible = false;
            recalculateAyahTextPointSize.start();
        }
        onHeightChanged: {
            ayahText.visible = false;
            recalculateAyahTextPointSize.start();
        }

        Label {
            id: ayahText
            anchors.horizontalCenter: parent.horizontalCenter
            anchors.verticalCenter: parent.verticalCenter
            width: ayahTextContainer.width

            horizontalAlignment: Text.AlignHCenter

            padding: 5
            wrapMode: Text.Wrap

            onTextChanged: {
                visible = false;
                recalculateAyahTextPointSize.start();
            }
        }
    }

    Label {
        id: quranError
        anchors.left: parent.left
        anchors.right: close.left
        anchors.verticalCenter: close.verticalCenter
        anchors.leftMargin: 7
        anchors.rightMargin: 7
        font.bold: true
        font.italic: true
        color: "#7A5544"
    }

    Button {
        id: close
        anchors.bottom: parent.bottom
        anchors.bottomMargin: 7
        anchors.right: parent.right
        anchors.rightMargin: 7
        text: qsTr("\u2190")
        onClicked: closePressed();

        background: Rectangle {
            color: close.pressed || !close.enabled ? Qt.rgba(1, 1, 1, .33) : Qt.rgba(1, 1, 1, .66)
            radius: 4
            border.color: "#000000"
        }
    }

    BusyIndicator {
        id: quranBusy
        anchors.fill: ayahTextContainer
        visible: false
    }
}
