import QtQuick 2.12
import QtQuick.Window 2.12
import QtQuick.Controls 2.3
import QtQuick.Dialogs 1.3
import QtQuick.Layouts 1.12

Item {
    signal checkForUpdatesButtonPressed();
    signal shutDownButtonPressed();
    signal countrySelected(string geonameId, string countryCode);
    signal customLocationSearchPressed(string countryGeonameId, string countryCode, string regionGeonameId, string regionCode, string searchText);
    signal settingsSubmitPressed(string settingsJSON);
    signal settingsCancelPressed();
    signal soundTestPressed(int event, int soundId);


    function setBkgDirCtrlVisible(isVisible)
    {
        bkgDirCtrl.visible = isVisible;
    }

    function setCheckForUpdatesButtonVisible(isVisible)
    {
        updateButton.visible   = isVisible;
        shutDownButton.visible = isVisible;
    }

    function setCountries(countries)
    {
        var selIndex   = -1;
        var textHeight = 0;

        countriesModel.clear();
        for (var i = 0; i < countries.countries.length; i++) {
            var newCountry = {
                geonameId: countries.countries[i].geoname_id,
                name     : countries.countries[i].name,
                code     : countries.countries[i].code
            };
            countriesModel.append(newCountry);

            if (countries.countries[i].code === countries.sel_code) {
                selIndex = i;
            }

            textMetrics.text = countries.countries[i].name;
            if (textMetrics.height > textHeight) {
                textHeight = textMetrics.height;
            }
        }

        if (selIndex >= 0) {
            locationCountry.currentIndex = selIndex;
        }
        if (textHeight > 0) {
            locationCountry.background.implicitHeight  = textHeight;
            locationCountry.contentItem.implicitHeight = textHeight;
            locationCountry.indicator.height           = textHeight;
        }
    }

    function setErrorStr(errorStr)
    {
        settingsError.text = errorStr;
    }

    function setFont(latinFamily, arabicFamily, size, prayersArabic, settingsArabic)
    {
        var familyToUse = settingsArabic ? arabicFamily : latinFamily;
        var sizeToUse   = (settingsArabic && arabicFamily) ? size * 1.2 : size;

        tabs.font.pointSize        = sizeToUse;
        textMetrics.font.pointSize = sizeToUse;

        languageLabel.font.pointSize  = sizeToUse;
        language.font.pointSize       = sizeToUse;
        language.popup.font.pointSize = sizeToUse;
        volumeLabel.font.pointSize    = sizeToUse;
        volume.font.pointSize         = sizeToUse;
        splashSound.font.pointSize    = sizeToUse;
        useBkg.font.pointSize         = sizeToUse;
        bkgDirLabel.font.pointSize    = sizeToUse;
        bkgDir.font.pointSize         = sizeToUse;
        updateButton.font.pointSize   = sizeToUse;
        shutDownButton.font.pointSize = sizeToUse;

        autoLocation.font.pointSize            = sizeToUse;
        customLocation.font.pointSize          = sizeToUse;
        locationCountry.font.pointSize         = sizeToUse;
        locationCountry.popup.font.pointSize   = sizeToUse;
        locationRegion.font.pointSize          = sizeToUse;
        locationRegion.popup.font.pointSize    = sizeToUse;
        locationSearch.font.pointSize          = sizeToUse;
        locationSearchButton.font.pointSize    = sizeToUse;

        customLocationsDelegateProperties.fontSize = sizeToUse

        dateShowGregorgian.font.pointSize         = sizeToUse;
        timeShowWesternArabic.font.pointSize      = sizeToUse;
        dateShowHijri.font.pointSize              = sizeToUse;
        timeShowEasternArabic.font.pointSize      = sizeToUse;
        dateShowShamsi.font.pointSize             = sizeToUse;
        timeShowFarsi.font.pointSize              = sizeToUse;
        dateShowFull.font.pointSize               = sizeToUse;
        time24.font.pointSize                     = sizeToUse;
        dateGregorianAlwaysEnglish.font.pointSize = sizeToUse;

        prayerNameOptionsLabel.font.pointSize           = sizeToUse;
        prayerNameOptions.font.pointSize                = sizeToUse;
        prayerNameOptions.popup.font.pointSize          = sizeToUse;
        prayerNameSubh.font.pointSize                   = sizeToUse;
        lastPrayerHighlightMinutesLabel.font.pointSize  = sizeToUse;
        lastPrayerHighlightMinutes.font.pointSize       = sizeToUse;
        nextPrayerHighlightMinutesLabel.font.pointSize  = sizeToUse;
        nextPrayerHighlightMinutes.font.pointSize       = sizeToUse;
        nextShurooqHighlightMinutesLabel.font.pointSize = sizeToUse;
        nextShurooqHighlightMinutes.font.pointSize      = sizeToUse;

        fajrSoundLabel.font.pointSize     = sizeToUse;
        fajrSound.font.pointSize          = sizeToUse;
        fajrSound.popup.font.pointSize    = sizeToUse;
        fajrTest.font.pointSize           = sizeToUse;
        shurooqSoundLabel.font.pointSize  = sizeToUse;
        shurooqSound.font.pointSize       = sizeToUse;
        shurooqSound.popup.font.pointSize = sizeToUse;
        shurooqTest.font.pointSize        = sizeToUse;
        dhuhrSoundLabel.font.pointSize    = sizeToUse;
        dhuhrSound.font.pointSize         = sizeToUse;
        dhuhrSound.popup.font.pointSize   = sizeToUse;
        dhuhrTest.font.pointSize          = sizeToUse;
        asrSoundLabel.font.pointSize      = sizeToUse;
        asrSound.font.pointSize           = sizeToUse;
        asrSound.popup.font.pointSize     = sizeToUse;
        asrTest.font.pointSize            = sizeToUse;
        maghribSoundLabel.font.pointSize  = sizeToUse;
        maghribSound.font.pointSize       = sizeToUse;
        maghribSound.popup.font.pointSize = sizeToUse;
        maghribTest.font.pointSize        = sizeToUse;
        ishaSoundLabel.font.pointSize     = sizeToUse;
        ishaSound.font.pointSize          = sizeToUse;
        ishaSound.popup.font.pointSize    = sizeToUse;
        ishaTest.font.pointSize           = sizeToUse;

        calcMethodLabel.font.pointSize  = sizeToUse;
        calcMethod.font.pointSize       = sizeToUse;
        calcMethod.popup.font.pointSize = sizeToUse;
        calcSchoolLabel.font.pointSize  = sizeToUse;
        calcSchool.font.pointSize       = sizeToUse;
        calcSchool.popup.font.pointSize = sizeToUse;
//      calcDetails.font.pointSize      = (settingsArabic && arabicFamily) ? size : size * .8;
        highLatLabel.font.pointSize     = sizeToUse;
        highLat.font.pointSize          = sizeToUse;
        highLat.popup.font.pointSize    = sizeToUse;
        extremeLatLabel.font.pointSize  = sizeToUse;
        extremeLat.font.pointSize       = sizeToUse;
        extremeLat.popup.font.pointSize = sizeToUse;
//      latDetails.font.pointSize       = (settingsArabic && arabicFamily) ? size : size * .8;

        alhamdulillah.font.pointSize = arabicFamily ? size * 3.1 : size * 2.9;
        appName.font.pointSize       = size * 2.9;
        version.font.pointSize       = sizeToUse;
        copyright.font.pointSize     = sizeToUse;
        website.font.pointSize       = sizeToUse;
        legal.font.pointSize         = (settingsArabic && arabicFamily) ? size : size * .8;
        qtlicense.font.pointSize     = (settingsArabic && arabicFamily) ? size : size * .8;

        settingsError.font.pointSize = size * .8;
        settingsError.font.family    = latinFamily;

        submit.font.pointSize = sizeToUse;
        cancel.font.pointSize = sizeToUse;

        if (familyToUse) {
            tabs.font.family        = familyToUse;
            textMetrics.font.family = familyToUse;

            languageLabel.font.family  = familyToUse;
            language.font.family       = familyToUse;
            language.popup.font.family = familyToUse;
            volumeLabel.font.family    = familyToUse;
            volume.font.family         = familyToUse;
            splashSound.font.family    = familyToUse;
            useBkg.font.family         = familyToUse;
            bkgDirLabel.font.family    = familyToUse;
            bkgDir.font.family         = familyToUse;
            updateButton.font.family   = familyToUse;
            shutDownButton.font.family = familyToUse;

            autoLocation.font.family            = familyToUse;
            customLocation.font.family          = familyToUse;
            locationCountry.font.family         = familyToUse;
            locationCountry.popup.font.family   = familyToUse;
            locationRegion.font.family          = familyToUse;
            locationRegion.popup.font.family    = familyToUse;
            locationSearch.font.family          = familyToUse;
            locationSearchButton.font.family    = familyToUse;

            customLocationsDelegateProperties.fontFamily = familyToUse;

            dateShowGregorgian.font.family         = familyToUse;
            timeShowWesternArabic.font.family      = familyToUse;
            dateShowHijri.font.family              = familyToUse;
            timeShowEasternArabic.font.family      = familyToUse;
            dateShowShamsi.font.family             = familyToUse;
            timeShowFarsi.font.family              = familyToUse;
            dateShowFull.font.family               = familyToUse;
            time24.font.family                     = familyToUse;
            dateGregorianAlwaysEnglish.font.family = familyToUse;

            prayerNameOptionsLabel.font.family           = familyToUse;
            prayerNameOptions.font.family                = familyToUse;
            prayerNameOptions.popup.font.family          = familyToUse;
            prayerNameSubh.font.family                   = familyToUse;
            lastPrayerHighlightMinutesLabel.font.family  = familyToUse;
            lastPrayerHighlightMinutes.font.family       = familyToUse;
            nextPrayerHighlightMinutesLabel.font.family  = familyToUse;
            nextPrayerHighlightMinutes.font.family       = familyToUse;
            nextShurooqHighlightMinutesLabel.font.family = familyToUse;
            nextShurooqHighlightMinutes.font.family      = familyToUse;

            fajrSoundLabel.font.family     = familyToUse;
            fajrSound.font.family          = familyToUse;
            fajrSound.popup.font.family    = familyToUse;
            fajrTest.font.family           = familyToUse;
            shurooqSoundLabel.font.family  = familyToUse;
            shurooqSound.font.family       = familyToUse;
            shurooqSound.popup.font.family = familyToUse;
            shurooqTest.font.family        = familyToUse;
            dhuhrSoundLabel.font.family    = familyToUse;
            dhuhrSound.font.family         = familyToUse;
            dhuhrSound.popup.font.family   = familyToUse;
            dhuhrTest.font.family          = familyToUse;
            asrSoundLabel.font.family      = familyToUse;
            asrSound.font.family           = familyToUse;
            asrSound.popup.font.family     = familyToUse;
            asrTest.font.family            = familyToUse;
            maghribSoundLabel.font.family  = familyToUse;
            maghribSound.font.family       = familyToUse;
            maghribSound.popup.font.family = familyToUse;
            maghribTest.font.family        = familyToUse;
            ishaSoundLabel.font.family     = familyToUse;
            ishaSound.font.family          = familyToUse;
            ishaSound.popup.font.family    = familyToUse;
            ishaTest.font.family           = familyToUse;

            calcMethodLabel.font.family  = familyToUse;
            calcMethod.font.family       = familyToUse;
            calcMethod.popup.font.family = familyToUse;
            calcSchoolLabel.font.family  = familyToUse;
            calcSchool.font.family       = familyToUse;
            calcSchool.popup.font.family = familyToUse;
//          calcDetails.font.family      = familyToUse;
            highLatLabel.font.family     = familyToUse;
            highLat.font.family          = familyToUse;
            highLat.popup.font.family    = familyToUse;
            extremeLatLabel.font.family  = familyToUse;
            extremeLat.font.family       = familyToUse;
            extremeLat.popup.font.family = familyToUse;
//          latDetails.font.family       = familyToUse;

            alhamdulillah.font.family = familyToUse;
            appName.font.family       = familyToUse;
            version.font.family       = familyToUse;
            copyright.font.family     = familyToUse;
            website.font.family       = familyToUse;
            legal.font.family         = familyToUse;
            qtlicense.font.family     = familyToUse;

            submit.font.family = familyToUse;
            cancel.font.family = familyToUse;
        }

        if (arabicFamily) {
            alhamdulillah.font.family = arabicFamily;
        }

        if (!settingsArabic && prayersArabic && arabicFamily) {
            fajrSoundLabel.font.pointSize    = size * 1.2;
            shurooqSoundLabel.font.pointSize = size * 1.2;
            dhuhrSoundLabel.font.pointSize   = size * 1.2;
            asrSoundLabel.font.pointSize     = size * 1.2;
            maghribSoundLabel.font.pointSize = size * 1.2;
            ishaSoundLabel.font.pointSize    = size * 1.2;

            fajrSoundLabel.font.family    = arabicFamily;
            shurooqSoundLabel.font.family = arabicFamily;
            dhuhrSoundLabel.font.family   = arabicFamily;
            asrSoundLabel.font.family     = arabicFamily;
            maghribSoundLabel.font.family = arabicFamily;
            ishaSoundLabel.font.family    = arabicFamily;
        }

        volume.handle.implicitHeight = 20;
        volume.handle.implicitWidth  = 20;

        var textHeight;
        var textWidth;

        var toBeMeasured = [
            [customLocation, autoLocation],
            [splashSound, useBkg],
            [dateShowGregorgian, dateShowHijri, dateShowShamsi, dateShowFull, dateGregorianAlwaysEnglish, timeShowWesternArabic, timeShowEasternArabic, timeShowFarsi, time24],
        ];
        for (var i = 0; i < toBeMeasured.length; i++) {
            textHeight = 0;

            for (var j = 0; j < toBeMeasured[i].length; j++) {
                textMetrics.text = toBeMeasured[i][j].text;
                if (textMetrics.height > textHeight) {
                    textHeight = textMetrics.height;
                }
            }

            for (j = 0; j < toBeMeasured[i].length; j++) {
                toBeMeasured[i][j].indicator.implicitWidth  = textHeight;
                toBeMeasured[i][j].indicator.implicitHeight = textHeight;
            }
        }

        textMetrics.text = cancel.text;
        textHeight       = textMetrics.height;
        textMetrics.text = submit.text;
        if (textMetrics.height > textHeight) {
            textHeight = textMetrics.height;
        }
        cancel.background.implicitHeight = textHeight;
        submit.background.implicitHeight = textHeight;

        textMetrics.text = locationSearchButton.text;
        locationSearchButton.background.implicitHeight = textMetrics.height;

        textMetrics.text = updateButton.text;
        updateButton.background.implicitHeight   = textMetrics.height;
        shutDownButton.background.implicitHeight = textMetrics.height;
    }

    function setLocationBusy(busy)
    {
        locationBusy.visible = busy;
    }

    function setRegions(regions)
    {
        var textHeight = 0;

        regionsModel.clear();
        for (var i = 0; i < regions.length; i++) {
            var newRegion = {
                geonameId: regions[i].geoname_id,
                name     : regions[i].name,
                code     : regions[i].code
            };
            regionsModel.append(newRegion);

            textMetrics.text = regions[i].name;
            if (textMetrics.height > textHeight) {
                textHeight = textMetrics.height;
            }
        }

        locationRegion.currentIndex = 0;
        if (textHeight > 0) {
            locationRegion.background.implicitHeight  = textHeight;
            locationRegion.contentItem.implicitHeight = textHeight;
            locationRegion.indicator.height           = textHeight;
        }
    }

    function setSearchResults(results)
    {
        var textHeight = 0;

        customLocationsModel.clear();
        for (var i = 0; i < results.length; i++) {
            var country = results[i].country;

            var j = 0;
            while (!country && (j < countriesModel.count)) {
                if (countriesModel.get(j).code === results[i].country_code) {
                    country = countriesModel.get(j).name;
                }
                j++;
            }

            var newResult = {
                city        : results[i].city,
                countryCode : results[i].country_code,
                country     : country,
                region      : results[i].region,
                latitude    : results[i].latitude,
                longitude   : results[i].longitude,
                timeZone    : results[i].time_zone,
                timeZoneCode: results[i].time_zone_code,
                displayStr  : results[i].display_str
            };
            customLocationsModel.append(newResult);

            textMetrics.text = results[i].display_str;
            if (textMetrics.height > textHeight) {
                textHeight = textMetrics.height;
            }
        }

        if (textHeight > 0) {
            customLocationsDelegateProperties.height = textHeight;
        }

        locationSearchButton.enabled = true;
    }

    function settingsInit(optionsMap)
    {
        var selIndex;
        var textHeight;

        selIndex   = -1;
        textHeight = 0;
        languagesModel.clear();
        for (var i = 0; i < optionsMap.languages.length; i++) {
            var newLanguage = {
                abbr: optionsMap.languages[i].abbr,
                name: optionsMap.languages[i].name
            };
            languagesModel.append(newLanguage);

            if (optionsMap.languages[i].abbr === optionsMap.language) {
                selIndex = i;
            }

            textMetrics.text = optionsMap.languages[i].name;
            if (textMetrics.height > textHeight) {
                textHeight = textMetrics.height;
            }
        }
        if (selIndex >= 0) {
            language.currentIndex = selIndex;
        }
        if (textHeight > 0) {
            language.background.implicitHeight  = textHeight;
            language.contentItem.implicitHeight = textHeight;
            language.indicator.height           = textHeight;
        }

        volume.value        = optionsMap.volume;
        splashSound.checked = optionsMap.splash_sound;
        useBkg.checked      = optionsMap.use_bkg;
        bkgDir.text         = optionsMap.bkg_dir;

        if (optionsMap.location_custom) {
            customLocation.checked = true;
        }
        else {
            autoLocation.checked = true;
        }
        setSearchResults(optionsMap.locations);

        dateShowGregorgian.checked         = optionsMap.gregorian_date_visible;
        dateShowHijri.checked              = optionsMap.hijri_date_visible;
        dateShowShamsi.checked             = optionsMap.shamsi_date_visible;
        dateShowFull.checked               = optionsMap.date_format_full;
        dateGregorianAlwaysEnglish.checked = optionsMap.gregorian_date_always_english;
        timeShowWesternArabic.checked      = optionsMap.western_arabic_time_visible;
        timeShowEasternArabic.checked      = optionsMap.eastern_arabic_time_visible;
        timeShowFarsi.checked              = optionsMap.farsi_time_visible;
        time24.font.checked                = optionsMap.time_format_24;

        selIndex   = -1;
        textHeight = 0;
        prayerNameOptionsModel.clear();
        for (i = 0; i < optionsMap.prayer_time_names_list.length; i++) {
            var newPrayerNamesOption = {
                id:   optionsMap.prayer_time_names_list[i].id,
                name: optionsMap.prayer_time_names_list[i].name
            };
            prayerNameOptionsModel.append(newPrayerNamesOption);

            if (optionsMap.prayer_time_names_list[i].id === optionsMap.prayer_times_names) {
                selIndex = i;
            }

            textMetrics.text = optionsMap.prayer_time_names_list[i].name;
            if (textMetrics.height > textHeight) {
                textHeight = textMetrics.height;
            }
        }
        if (selIndex >= 0) {
            prayerNameOptions.currentIndex = selIndex;
        }
        if (textHeight > 0) {
            prayerNameOptions.background.implicitHeight  = textHeight;
            prayerNameOptions.contentItem.implicitHeight = textHeight;
            prayerNameOptions.indicator.height           = textHeight;
        }

        prayerNameSubh.checked            = optionsMap.prayer_times_subh;
        lastPrayerHighlightMinutes.value  = optionsMap.last_prayer_highlight_minutes;
        nextPrayerHighlightMinutes.value  = optionsMap.next_prayer_highlight_minutes;
        nextShurooqHighlightMinutes.value = optionsMap.next_shurooq_highlight_minutes;

        textMetrics.text = optionsMap.last_prayer_highlight_minutes;

        lastPrayerHighlightMinutes.background.implicitHeight     = textMetrics.height;
        lastPrayerHighlightMinutes.up.indicator.implicitHeight   = textMetrics.height;
        lastPrayerHighlightMinutes.down.indicator.implicitHeight = textMetrics.height;

        nextPrayerHighlightMinutes.background.implicitHeight     = textMetrics.height;
        nextPrayerHighlightMinutes.up.indicator.implicitHeight   = textMetrics.height;
        nextPrayerHighlightMinutes.down.indicator.implicitHeight = textMetrics.height;

        nextShurooqHighlightMinutes.background.implicitHeight     = textMetrics.height;
        nextShurooqHighlightMinutes.up.indicator.implicitHeight   = textMetrics.height;
        nextShurooqHighlightMinutes.down.indicator.implicitHeight = textMetrics.height;

        textMetrics.text = prayerNameSubh.text;
        prayerNameSubh.indicator.implicitWidth  = textMetrics.height;
        prayerNameSubh.indicator.implicitHeight = textMetrics.height;

        fajrSoundLabel.text    = optionsMap.fajr
        shurooqSoundLabel.text = optionsMap.shurooq
        dhuhrSoundLabel.text   = optionsMap.dhuhr
        asrSoundLabel.text     = optionsMap.asr
        maghribSoundLabel.text = optionsMap.maghrib
        ishaSoundLabel.text    = optionsMap.isha

        var fajrSelected    = -1;
        var dhuhrSelected   = -1;
        var asrSelected     = -1;
        var maghribSelected = -1;
        var ishaSelected    = -1;
        textHeight = 0;
        prayerSoundModel.clear();
        for (i = 0; i < optionsMap.prayer_sounds.length; i++) {
            var newPrayerSound = {
                id  : optionsMap.prayer_sounds[i].id,
                name: optionsMap.prayer_sounds[i].name
            };
            prayerSoundModel.append(newPrayerSound);

            if (optionsMap.prayer_sounds[i].id === optionsMap.sound_fajr) {
                fajrSelected = i;
            }
            if (optionsMap.prayer_sounds[i].id === optionsMap.sound_dhuhr) {
                dhuhrSelected = i;
            }
            if (optionsMap.prayer_sounds[i].id === optionsMap.sound_asr) {
                asrSelected = i;
            }
            if (optionsMap.prayer_sounds[i].id === optionsMap.sound_maghrib) {
                maghribSelected = i;
            }
            if (optionsMap.prayer_sounds[i].id === optionsMap.sound_isha) {
                ishaSelected = i;
            }

            textMetrics.text = optionsMap.prayer_sounds[i].name;
            if (textMetrics.height > textHeight) {
                textHeight = textMetrics.height;
            }
        }
        if (fajrSelected >= 0) {
            fajrSound.currentIndex = fajrSelected;
        }
        if (dhuhrSelected >= 0) {
            dhuhrSound.currentIndex = dhuhrSelected;
        }
        if (asrSelected >= 0) {
            asrSound.currentIndex = asrSelected;
        }
        if (maghribSelected >= 0) {
            maghribSound.currentIndex = maghribSelected;
        }
        if (ishaSelected >= 0) {
            ishaSound.currentIndex = ishaSelected;
        }

        selIndex = -1;
        shurooqSoundModel.clear();
        for (i = 0; i < optionsMap.shurooq_sounds.length; i++) {
            var newShurooqSound = {
                id  : optionsMap.shurooq_sounds[i].id,
                name: optionsMap.shurooq_sounds[i].name
            };
            shurooqSoundModel.append(newShurooqSound);

            if (optionsMap.shurooq_sounds[i].id === optionsMap.sound_shurooq) {
                selIndex = i;
            }

            textMetrics.text = optionsMap.shurooq_sounds[i].name;
            if (textMetrics.height > textHeight) {
                textHeight = textMetrics.height;
            }
        }
        if (selIndex >= 0) {
            shurooqSound.currentIndex = selIndex;
        }

        if (textHeight > 0) {
            fajrSound.background.implicitHeight  = textHeight;
            fajrSound.contentItem.implicitHeight = textHeight;
            fajrSound.indicator.height           = textHeight;

            shurooqSound.background.implicitHeight  = textHeight;
            shurooqSound.contentItem.implicitHeight = textHeight;
            shurooqSound.indicator.height           = textHeight;

            dhuhrSound.background.implicitHeight  = textHeight;
            dhuhrSound.contentItem.implicitHeight = textHeight;
            dhuhrSound.indicator.height           = textHeight;

            asrSound.background.implicitHeight  = textHeight;
            asrSound.contentItem.implicitHeight = textHeight;
            asrSound.indicator.height           = textHeight;

            maghribSound.background.implicitHeight  = textHeight;
            maghribSound.contentItem.implicitHeight = textHeight;
            maghribSound.indicator.height           = textHeight;

            ishaSound.background.implicitHeight  = textHeight;
            ishaSound.contentItem.implicitHeight = textHeight;
            ishaSound.indicator.height           = textHeight;
        }

        textHeight  = 0;
        selIndex    = -1;
        calcMethodModel.clear();
        for (i = 0; i < optionsMap.calculation_methods.length; i++) {
            var newCalcMethod = {
                id  : optionsMap.calculation_methods[i].id,
                name: optionsMap.calculation_methods[i].name
            };
            calcMethodModel.append(newCalcMethod);

            if (optionsMap.calculation_methods[i].id === optionsMap.prayer_times_method) {
                selIndex = i;
            }

            textMetrics.text = optionsMap.calculation_methods[i].name;
            if (textMetrics.height > textHeight) {
                textHeight = textMetrics.height;
            }
        }
        if (selIndex >= 0) {
            calcMethod.currentIndex = selIndex;
        }

        selIndex = -1;
        calcSchoolModel.clear();
        for (i = 0; i < optionsMap.calculation_schools.length; i++) {
            var newCalcSchool = {
                id  : optionsMap.calculation_schools[i].id,
                name: optionsMap.calculation_schools[i].name
            };
            calcSchoolModel.append(newCalcSchool);

            if (optionsMap.calculation_schools[i].id === optionsMap.prayer_times_school) {
                selIndex = i;
            }

            textMetrics.text = optionsMap.calculation_schools[i].name;
            if (textMetrics.height > textHeight) {
                textHeight = textMetrics.height;
            }
        }
        if (selIndex >= 0) {
            calcSchool.currentIndex = selIndex;
        }

        selIndex = -1;
        highLatModel.clear();
        for (i = 0; i < optionsMap.high_latitudes.length; i++) {
            var newHighLat = {
                id  : optionsMap.high_latitudes[i].id,
                name: optionsMap.high_latitudes[i].name
            };
            highLatModel.append(newHighLat);

            if (optionsMap.high_latitudes[i].id === optionsMap.prayer_times_high_latitude) {
                selIndex = i;
            }

            textMetrics.text = optionsMap.high_latitudes[i].name;
            if (textMetrics.height > textHeight) {
                textHeight = textMetrics.height;
            }
        }
        if (selIndex >= 0) {
            highLat.currentIndex = selIndex;
        }

        selIndex = -1;
        extremeLatModel.clear();
        for (i = 0; i < optionsMap.extreme_latitudes.length; i++) {
            var newExtremeMethod = {
                id  : optionsMap.extreme_latitudes[i].id,
                name: optionsMap.extreme_latitudes[i].name
            };
            extremeLatModel.append(newExtremeMethod);

            if (optionsMap.extreme_latitudes[i].id === optionsMap.prayer_times_extreme_latitude) {
                selIndex = i;
            }

            textMetrics.text = optionsMap.extreme_latitudes[i].name;
            if (textMetrics.height > textHeight) {
                textHeight = textMetrics.height;
            }
        }
        if (selIndex >= 0) {
            extremeLat.currentIndex = selIndex;
        }

        if (textHeight > 0) {
            calcMethod.background.implicitHeight  = textHeight;
            calcMethod.contentItem.implicitHeight = textHeight;
            calcMethod.indicator.height           = textHeight;

            calcSchool.background.implicitHeight  = textHeight;
            calcSchool.contentItem.implicitHeight = textHeight;
            calcSchool.indicator.height           = textHeight;

            highLat.background.implicitHeight  = textHeight;
            highLat.contentItem.implicitHeight = textHeight;
            highLat.indicator.height           = textHeight;

            extremeLat.background.implicitHeight  = textHeight;
            extremeLat.contentItem.implicitHeight = textHeight;
            extremeLat.indicator.height           = textHeight;
        }

        version.text = optionsMap.version;
    }


    QtObject {
        id: customLocationsDelegateProperties

        property int    fontSize  : 8
        property string fontFamily: "Sans"
        property int    height    : 16
    }

    ListModel {
        id: languagesModel
    }
    ListModel {
        id: countriesModel
    }
    ListModel {
        id: regionsModel
    }
    ListModel {
        id: customLocationsModel
    }
    ListModel {
        id: prayerNameOptionsModel
    }
    ListModel {
        id: prayerSoundModel
    }
    ListModel {
        id: shurooqSoundModel
    }
    ListModel {
        id: calcMethodModel
    }
    ListModel {
        id: calcSchoolModel
    }
    ListModel {
        id: highLatModel
    }
    ListModel {
        id: extremeLatModel
    }

    TextMetrics {
        id: textMetrics
    }


    TabBar {
        id: tabs
        width: parent.width

        TabButton {
            text: qsTr("General")
        }
        TabButton {
            text: qsTr("Location")
        }
        TabButton {
            text: qsTr("Time")
        }
        TabButton {
            text: qsTr("Prayers")
        }
        TabButton {
            text: qsTr("Sound")
        }
        TabButton {
            text: qsTr("Calculation")
            width: implicitWidth
        }
        TabButton {
            text: qsTr("About")
        }
    }

    StackLayout {
        anchors.top: tabs.bottom
        anchors.bottom: submit.top
        anchors.left: parent.left
        anchors.right: parent.right
        anchors.margins: 3
        currentIndex: tabs.currentIndex

        Flickable {
            x: 0
            y: 0
            width: parent.width
            height: parent.height
            clip: true
            contentHeight: general.height

            ScrollBar.vertical: ScrollBar { }

            Column {
                id: general
                spacing: 3
                width: parent.width

                Row {
                    spacing: 3

                    Label {
                        id: languageLabel
                        anchors.verticalCenter: language.verticalCenter
                        text: qsTr("Language")
                        width: parent.parent.width / 4
                        wrapMode: Label.WrapAtWordBoundaryOrAnywhere
                    }
                    ComboBox {
                        id: language
                        editable: false
                        model: languagesModel
                        textRole: "name"
                        width: parent.parent.width / 4 * 3
                    }
                }
                Row {
                    spacing: 3

                    Label {
                        id: volumeLabel
                        anchors.verticalCenter: volume.verticalCenter
                        text: qsTr("Volume")
                        width: parent.parent.width / 4
                        wrapMode: Label.WrapAtWordBoundaryOrAnywhere
                    }
                    Slider {
                        id: volume
                        from: 0
                        to: 100
                        width: parent.parent.width / 4 * 3
                    }
                }
                Row {
                    spacing: 3

                    CheckBox {
                        id: splashSound
                        text: qsTr("Play sound at start")
                    }
                }
                Row {
                    spacing: 3

                    CheckBox {
                        id: useBkg
                        text: qsTr("Show background images")
                    }
                }
                Row {
                    id: bkgDirCtrl
                    spacing: 3
                    visible: false

                    Label {
                        id: bkgDirLabel
                        anchors.verticalCenter: bkgDir.verticalCenter
                        text: qsTr("Background images")
                        width: implicitWidth
                        wrapMode: Label.WrapAtWordBoundaryOrAnywhere
                    }
                    TextField {
                        id: bkgDir
                        onPressed: dirDialog.open()
                        readOnly: true
                        width: parent.parent.width - bkgDirLabel.width - 3
                    }
                }
                Row {
                    spacing: 3
                    topPadding: 5
                    layoutDirection: Qt.RightToLeft
                    width: parent.width

                    Button {
                        id: updateButton
                        visible: false
                        text: qsTr("Check for updates")
                        onClicked: checkForUpdatesButtonPressed();
                    }

                    Button {
                        id: shutDownButton
                        visible: false
                        text: qsTr("Shut down")
                        onClicked: shutDownButtonPressed();
                    }
                }
            }
        }

        Flickable {
            x: 0
            y: 0
            width: parent.width
            height: parent.height
            clip: true
            contentHeight: location.height

            ScrollBar.vertical: ScrollBar { }

            Column {
                id: location
                spacing: 3
                width: parent.width

                Row {
                    spacing: 3

                    RadioButton {
                        id: autoLocation
                        text: qsTr("Automatic")
                    }
                    RadioButton {
                        id: customLocation
                        enabled: countriesModel.count > 0
                        text: qsTr("Custom")
                    }
                }

                Row {
                    spacing: 3

                    ComboBox {
                        id: locationCountry
                        enabled: customLocation.checked && (countriesModel.count > 0)
                        model: countriesModel
                        textRole: "name"
                        width: parent.parent.width

                        onCurrentIndexChanged: {
                            regionsModel.clear();
                            countrySelected(countriesModel.get(currentIndex).geonameId, countriesModel.get(currentIndex).code);
                        }
                    }
                }
                Row {
                    spacing: 3

                    ComboBox {
                        id: locationRegion
                        enabled: locationCountry.enabled && (regionsModel.count > 0)
                        model: regionsModel
                        textRole: "name"
                        width: parent.parent.width
                    }
                }
                Row {
                    spacing: 3

                    TextField {
                        id: locationSearch
                        enabled: locationCountry.enabled && (locationCountry.currentIndex >= 0)
                        placeholderText: enabled && (countriesModel.get(locationCountry.currentIndex).code === "US") ? qsTr("Search text or ZIP code") : qsTr("Search text or postal code")
                        width: parent.parent.width - locationSearchButton.width - 3
                    }
                    Button {
                        id: locationSearchButton
                        anchors.verticalCenter: locationSearch.verticalCenter
                        enabled: locationSearch.enabled && (locationSearch.text.length >= 3)
                        text: qsTr("Search")

                        onClicked: {
                            locationSearchButton.enabled = false;

                            var hasRegion = locationRegion.enabled && (locationRegion.currentIndex > 0);
                            customLocationSearchPressed(
                                countriesModel.get(locationCountry.currentIndex).geonameId,
                                countriesModel.get(locationCountry.currentIndex).code,
                                hasRegion ? regionsModel.get(locationRegion.currentIndex).geonameId : "",
                                hasRegion ? regionsModel.get(locationRegion.currentIndex).code : "",
                                locationSearch.text
                            );
                        }
                    }
                }
                Row {
                    spacing: 3

                    ListView {
                        id: customLocationsList
                        height: (customLocationsDelegateProperties.height + 6) * customLocationsModel.count
                        width: parent.parent.width
                        clip: true
                        delegate: Component {
                            id: customLocationsDelegate

                            MouseArea {
                                height: customLocationsDelegateProperties.height + 6
                                width: customLocationsList.width

                                acceptedButtons: Qt.LeftButton
                                onClicked: customLocationsList.currentIndex = index;

                                Label {
                                    font.family: customLocationsDelegateProperties.fontSize
                                    font.pointSize: customLocationsDelegateProperties.fontSize
                                    height: customLocationsDelegateProperties.height + 6
                                    padding: 3
                                    text: displayStr
                                    width: customLocationsList.width
                                }
                            }
                        }
                        highlight: Rectangle {
                            radius: 3
                            color: "LightSteelBlue";
                        }
                        model: customLocationsModel
                    }
                }
            }

            BusyIndicator {
                id: locationBusy
                anchors.horizontalCenter: parent.horizontalCenter
                y: parent.parent.height / 2
                width: location.width > 100 ? 100 : location.width - 20
                height: width
                visible: false
            }
        }

        Flickable {
            x: 0
            y: 0
            width: parent.width
            height: parent.height
            clip: true
            contentHeight: times.height

            ScrollBar.vertical: ScrollBar { }

            Column {
                id: times
                spacing: 3
                width: parent.width

                Row {
                    spacing: 3

                    CheckBox {
                        id: dateShowGregorgian
                        text: qsTr("Show Gregorian date")
                        width: parent.parent.width / 2
                    }
                    CheckBox {
                        id: timeShowWesternArabic
                        text: qsTr("Show Western Arabic time")
                    }
                }
                Row {
                    spacing: 3

                    CheckBox {
                        id: dateShowHijri
                        text: qsTr("Show Hijri date")
                        width: parent.parent.width / 2
                    }
                    CheckBox {
                        id: timeShowEasternArabic
                        text: qsTr("Show Eastern Arabic time")
                    }
                }
                Row {
                    spacing: 3

                    CheckBox {
                        id: dateShowShamsi
                        text: qsTr("Show Shamsi Hijri date")
                        width: parent.parent.width / 2
                    }
                    CheckBox {
                        id: timeShowFarsi
                        text: qsTr("Show Farsi time")
                    }
                }
                Row {
                    spacing: 3

                    CheckBox {
                        id: dateShowFull
                        text: qsTr("Long format date")
                        width: parent.parent.width / 2
                    }
                    CheckBox {
                        id: time24
                        text: qsTr("24 hour format time")
                    }
                }
                Row {
                    spacing: 3

                    CheckBox {
                        id: dateGregorianAlwaysEnglish
                        text: qsTr("Gregorian date always English")
                    }
                }
            }
        }

        Flickable {
            x: 0
            y: 0
            width: parent.width
            height: parent.height
            clip: true
            contentHeight: prayers.height

            ScrollBar.vertical: ScrollBar { }

            Column {
                id: prayers
                spacing: 3
                width: parent.width

                Row {
                    spacing: 3

                    Label {
                        id: prayerNameOptionsLabel
                        anchors.verticalCenter: prayerNameOptions.verticalCenter
                        text: qsTr("Prayer names")
                        width: parent.parent.width / 4 - 3
                        wrapMode: Label.WrapAtWordBoundaryOrAnywhere
                    }
                    ComboBox {
                        id: prayerNameOptions
                        editable: false
                        model: prayerNameOptionsModel
                        textRole: "name"
                        width: parent.parent.width / 4 * 2
                    }
                    CheckBox {
                        id: prayerNameSubh
                        anchors.verticalCenter: prayerNameOptions.verticalCenter
                        text: qsTr("Use Subh")
                        width: parent.parent.width / 4 - 3
                    }
                }
                Row {
                    spacing: 3

                    Label {
                        id: lastPrayerHighlightMinutesLabel
                        anchors.verticalCenter: lastPrayerHighlightMinutes.verticalCenter
                        text: qsTr("Prayer Highlight (minutes)")
                        width: parent.parent.width / 2
                        wrapMode: Label.WrapAtWordBoundaryOrAnywhere
                    }
                    SpinBox {
                        id: lastPrayerHighlightMinutes
                        from: 0
                        to: 30
                    }
                }
                Row {
                    spacing: 3

                    Label {
                        id: nextPrayerHighlightMinutesLabel
                        anchors.verticalCenter: nextPrayerHighlightMinutes.verticalCenter
                        text: qsTr("Prayer Countdown (minutes)")
                        width: parent.parent.width / 2
                        wrapMode: Label.WrapAtWordBoundaryOrAnywhere
                    }
                    SpinBox {
                        id: nextPrayerHighlightMinutes
                        from: 0
                        to: 15
                    }
                }
                Row {
                    spacing: 3

                    Label {
                        id: nextShurooqHighlightMinutesLabel
                        anchors.verticalCenter: nextShurooqHighlightMinutes.verticalCenter
                        text: qsTr("Sunrise Countdown (minutes)")
                        width: parent.parent.width / 2
                        wrapMode: Label.WrapAtWordBoundaryOrAnywhere
                    }
                    SpinBox {
                        id: nextShurooqHighlightMinutes
                        from: 0
                        to: 10
                    }
                }
            }
        }

        Flickable {
            x: 0
            y: 0
            width: parent.width
            height: parent.height
            clip: true
            contentHeight: sound.height

            ScrollBar.vertical: ScrollBar { }

            Column {
                id: sound
                spacing: 3
                width: parent.width

                Row {
                    spacing: 3

                    Label {
                        id: fajrSoundLabel
                        anchors.verticalCenter: fajrSound.verticalCenter
                        text: qsTr("Dawn prayer")
                        width: parent.parent.width / 4
                        wrapMode: Label.WrapAtWordBoundaryOrAnywhere
                    }
                    ComboBox {
                        id: fajrSound
                        editable: false
                        model: prayerSoundModel
                        textRole: "name"
                        width: parent.parent.width / 8 * 5
                    }
                    Button {
                        id: fajrTest
                        text: "\u25B6"
                        height: fajrSound.height
                        width: parent.parent.width / 8
                        onPressed: soundTestPressed(2, prayerSoundModel.get(fajrSound.currentIndex).id)
                    }
                }
                Row {
                    spacing: 3

                    Label {
                        id: shurooqSoundLabel
                        anchors.verticalCenter: shurooqSound.verticalCenter
                        text: qsTr("Sunrise")
                        width: parent.parent.width / 4
                        wrapMode: Label.WrapAtWordBoundaryOrAnywhere
                    }
                    ComboBox {
                        id: shurooqSound
                        editable: false
                        model: shurooqSoundModel
                        textRole: "name"
                        width: parent.parent.width / 8 * 5
                    }
                    Button {
                        id: shurooqTest
                        text: "\u25B6"
                        height: shurooqSound.height
                        width: parent.parent.width / 8
                        onPressed: soundTestPressed(3, shurooqSoundModel.get(shurooqSound.currentIndex).id)
                    }
                }
                Row {
                    spacing: 3

                    Label {
                        id: dhuhrSoundLabel
                        anchors.verticalCenter: dhuhrSound.verticalCenter
                        text: qsTr("Noon prayer")
                        width: parent.parent.width / 4
                        wrapMode: Label.WrapAtWordBoundaryOrAnywhere
                    }
                    ComboBox {
                        id: dhuhrSound
                        editable: false
                        model: prayerSoundModel
                        textRole: "name"
                        width: parent.parent.width / 8 * 5
                    }
                    Button {
                        id: dhuhrTest
                        text: "\u25B6"
                        height: dhuhrSound.height
                        width: parent.parent.width / 8
                        onPressed: soundTestPressed(4, prayerSoundModel.get(dhuhrSound.currentIndex).id)
                    }
                }
                Row {
                    spacing: 3

                    Label {
                        id: asrSoundLabel
                        anchors.verticalCenter: asrSound.verticalCenter
                        text: qsTr("Afternoon prayer")
                        width: parent.parent.width / 4
                        wrapMode: Label.WrapAtWordBoundaryOrAnywhere
                    }
                    ComboBox {
                        id: asrSound
                        editable: false
                        model: prayerSoundModel
                        textRole: "name"
                        width: parent.parent.width / 8 * 5
                    }
                    Button {
                        id: asrTest
                        text: "\u25B6"
                        height: asrSound.height
                        width: parent.parent.width / 8
                        onPressed: soundTestPressed(5, prayerSoundModel.get(asrSound.currentIndex).id)
                    }
                }
                Row {
                    spacing: 3

                    Label {
                        id: maghribSoundLabel
                        anchors.verticalCenter: maghribSound.verticalCenter
                        text: qsTr("Sunset prayer")
                        width: parent.parent.width / 4
                        wrapMode: Label.WrapAtWordBoundaryOrAnywhere
                    }
                    ComboBox {
                        id: maghribSound
                        editable: false
                        model: prayerSoundModel
                        textRole: "name"
                        width: parent.parent.width / 8 * 5
                    }
                    Button {
                        id: maghribTest
                        text: "\u25B6"
                        height: maghribSound.height
                        width: parent.parent.width / 8
                        onPressed: soundTestPressed(6, prayerSoundModel.get(maghribSound.currentIndex).id)
                    }
                }
                Row {
                    spacing: 3

                    Label {
                        id: ishaSoundLabel
                        anchors.verticalCenter: ishaSound.verticalCenter
                        text: qsTr("Night prayer")
                        width: parent.parent.width / 4
                        wrapMode: Label.WrapAtWordBoundaryOrAnywhere
                    }
                    ComboBox {
                        id: ishaSound
                        editable: false
                        model: prayerSoundModel
                        textRole: "name"
                        width: parent.parent.width / 8 * 5
                    }
                    Button {
                        id: ishaTest
                        text: "\u25B6"
                        height: ishaSound.height
                        width: parent.parent.width / 8
                        onPressed: soundTestPressed(7, prayerSoundModel.get(ishaSound.currentIndex).id)
                    }
                }
            }
        }

        Flickable {
            x: 0
            y: 0
            width: parent.width
            height: parent.height
            clip: true
            contentHeight: calculation.height

            ScrollBar.vertical: ScrollBar { }

            Column {
                id: calculation
                spacing: 3
                width: parent.width

                Row {
                    spacing: 3

                    Label {
                        id: calcMethodLabel
                        anchors.verticalCenter: calcMethod.verticalCenter
                        text: qsTr("Method")
                        width: parent.parent.width / 4
                        wrapMode: Label.WrapAtWordBoundaryOrAnywhere
                    }
                    ComboBox {
                        id: calcMethod
                        editable: false
                        model: calcMethodModel
                        textRole: "name"
                        width: parent.parent.width / 4 * 3
                    }
                }
                Row {
                    spacing: 3

                    Label {
                        id: calcSchoolLabel
                        anchors.verticalCenter: calcSchool.verticalCenter
                        text: qsTr("School of thought")
                        width: parent.parent.width / 4
                        wrapMode: Label.WrapAtWordBoundaryOrAnywhere
                    }
                    ComboBox {
                        id: calcSchool
                        editable: false
                        model: calcSchoolModel
                        textRole: "name"
                        width: parent.parent.width / 4 * 3
                    }
                }
                /*
                Row {
                    Label {
                        id: calcDetails
                    }
                }
                */
                Row {
                    spacing: 3

                    Label {
                        id: highLatLabel
                        anchors.verticalCenter: highLat.verticalCenter
                        text: qsTr("High latitude")
                        width: parent.parent.width / 4
                        wrapMode: Label.WrapAtWordBoundaryOrAnywhere
                    }
                    ComboBox {
                        id: highLat
                        editable: false
                        model: highLatModel
                        textRole: "name"
                        width: parent.parent.width / 4 * 3
                    }
                }
                Row {
                    spacing: 3

                    Label {
                        id: extremeLatLabel
                        anchors.verticalCenter: extremeLat.verticalCenter
                        text: qsTr("Extreme latitude")
                        width: parent.parent.width / 4
                        wrapMode: Label.WrapAtWordBoundaryOrAnywhere
                    }
                    ComboBox {
                        id: extremeLat
                        editable: false
                        model: extremeLatModel
                        textRole: "name"
                        width: parent.parent.width / 4 * 3
                    }
                }
                /*
                Row {
                    Label {
                        id: latDetails
                    }
                }
                */
            }
        }

        Flickable {
            x: 0
            y: 0
            width: parent.width
            height: parent.height
            clip: true
            contentHeight: about.height

            ScrollBar.vertical: ScrollBar { }

            Column {
                id: about
                spacing: 7
                width: parent.width

                Label {
                    id: alhamdulillah
                    anchors.horizontalCenter: parent.horizontalCenter
                    text: "الحمد لله رب العالمين"
                }
                Label {
                    id: appName
                    anchors.horizontalCenter: parent.horizontalCenter
                    font.bold: true
                    text: "Efendi Adhan Clock"
                }
                Label {
                    id: version
                    anchors.horizontalCenter: parent.horizontalCenter
                    text: "Version 0.0"
                }
                Label {
                    id: copyright
                    anchors.horizontalCenter: parent.horizontalCenter
                    text: "© 2024 The Efendi Factor"
                }
                Label {
                    id: website
                    anchors.horizontalCenter: parent.horizontalCenter
                    text: "<a href=\"https://efendif.com/\">efendif.com</a>"

                    onLinkActivated: Qt.openUrlExternally(link);

                    MouseArea {
                        anchors.fill: parent
                        acceptedButtons: Qt.NoButton
                        cursorShape: parent.hoveredLink ? Qt.PointingHandCursor : Qt.ArrowCursor
                    }
                }
                Label {
                    id: legal
                    horizontalAlignment: Text.AlignHCenter
                    text: "<br>This is free software: you can redistribute it and/or modify it under the
                           terms of the GNU General Public License as published by the Free Software
                           Foundation, either version 3 of the License, or (at your option) any later
                           version.<br><br>This software is distributed in the hope that it will be useful,
                           but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
                           or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
                           more details.<br><br><a href=\"https://www.gnu.org/licenses/gpl-3.0.en.html\">
                           GNU General Public License version 3</a><br>"
                    textFormat: Text.StyledText
                    width: parent.parent.width
                    wrapMode: Label.WrapAtWordBoundaryOrAnywhere

                    onLinkActivated: Qt.openUrlExternally(link);

                    MouseArea {
                        anchors.fill: parent
                        acceptedButtons: Qt.NoButton
                        cursorShape: parent.hoveredLink ? Qt.PointingHandCursor : Qt.ArrowCursor
                    }
                }
                Label {
                    id: qtlicense
                    horizontalAlignment: Text.AlignHCenter
                    text: "Efendi Adhan Clock is built on the
                           <a href=\"https://www.qt.io/\">Qt</a> framework. Qt is dual-licensed under
                           commercial and <a href=\"https://github.com/qt\">open source</a> licenses.
                           Version 5.12.8 Copyright (C) 2018 The Qt Company Ltd. Here, the
                           <a href=\"https://www.gnu.org/licenses/gpl-3.0.en.html\">GPL v3</a>
                           licensed version is being used, unmodified, by ways of dynamic linking.<br>"
                    textFormat: Text.StyledText
                    width: parent.width
                    wrapMode: Text.WrapAtWordBoundaryOrAnywhere

                    onLinkActivated: Qt.openUrlExternally(link);

                    MouseArea {
                        anchors.fill: parent
                        acceptedButtons: Qt.NoButton
                        cursorShape: parent.hoveredLink ? Qt.PointingHandCursor : Qt.ArrowCursor
                    }
                }
            }
        }

    }

    Label {
        id: settingsError
        anchors.left: parent.left
        anchors.right: cancel.left
        anchors.verticalCenter: cancel.verticalCenter
        anchors.leftMargin: 7
        anchors.rightMargin: 7
        font.bold: true
        font.italic: true
        color: "#7A5544"
    }

    FileDialog {
        id: dirDialog
        title: qsTr("Select media directory")
        folder: shortcuts.home
        selectFolder: true
        onAccepted: {
            bkgDir.text = folder.toString().replace("file://", "")
        }
    }

    Button {
        id: submit
        anchors.bottom: parent.bottom
        anchors.bottomMargin: 3
        anchors.right: parent.right
        anchors.rightMargin: 3
        text: qsTr("OK")
        onClicked: {
            var optionsObj = {
                language    : languagesModel.get(language.currentIndex).abbr,
                volume      : volume.value,
                splash_sound: splashSound.checked,
                use_bkg     : useBkg.checked,
                bkg_dir     : bkgDir.text,

                location_custom  : customLocation.checked,
                location_selected: customLocationsModel.get(customLocationsList.currentIndex),

                gregorian_date_visible       : dateShowGregorgian.checked,
                hijri_date_visible           : dateShowHijri.checked,
                shamsi_date_visible          : dateShowShamsi.checked,
                date_format_full             : dateShowFull.checked,
                gregorian_date_always_english: dateGregorianAlwaysEnglish.checked,
                western_arabic_time_visible  : timeShowWesternArabic.checked,
                eastern_arabic_time_visible  : timeShowEasternArabic.checked,
                farsi_time_visible           : timeShowFarsi.checked,
                time_format_24               : time24.checked,

                prayer_times_names            : prayerNameOptionsModel.get(prayerNameOptions.currentIndex).id,
                prayer_times_subh             : prayerNameSubh.checked,
                last_prayer_highlight_minutes : lastPrayerHighlightMinutes.value,
                next_prayer_highlight_minutes : nextPrayerHighlightMinutes.value,
                next_shurooq_highlight_minutes: nextShurooqHighlightMinutes.value,

                sound_fajr   : prayerSoundModel.get(fajrSound.currentIndex).id,
                sound_shurooq: shurooqSoundModel.get(shurooqSound.currentIndex).id,
                sound_dhuhr  : prayerSoundModel.get(dhuhrSound.currentIndex).id,
                sound_asr    : prayerSoundModel.get(asrSound.currentIndex).id,
                sound_maghrib: prayerSoundModel.get(maghribSound.currentIndex).id,
                sound_isha   : prayerSoundModel.get(ishaSound.currentIndex).id,

                prayer_times_method          : calcMethodModel.get(calcMethod.currentIndex).id,
                prayer_times_school          : calcSchoolModel.get(calcSchool.currentIndex).id,
                prayer_times_high_latitude   : highLatModel.get(highLat.currentIndex).id,
                prayer_times_extreme_latitude: extremeLatModel.get(extremeLat.currentIndex).id
            };

            settingsSubmitPressed(JSON.stringify(optionsObj));
        }
    }
    Button {
        id: cancel
        anchors.bottom: parent.bottom
        anchors.bottomMargin: 3
        anchors.right: submit.left
        anchors.rightMargin: 3
        text: qsTr("Cancel")
        onClicked: settingsCancelPressed();
    }
}
