import QtQuick 2.12
import QtQuick.Window 2.12
import QtQuick.Controls 2.3
import QtQuick.Dialogs 1.3
import QtQuick.Layouts 1.12

Item {
    signal wifiSelectPressed(string ssid, string psw);
    signal wifiCancelPressed();


    function setErrorStr(errorStr)
    {
        wifiError.text = errorStr;
    }


    function setFont(latinFamily, arabicFamily, size, useArabic)
    {
        wifiError.font.pointSize = size * .8;
        wifiError.font.family    = latinFamily;

        if (useArabic) {
            ssidDelegateProperties.fontSize   = size * 1.2;
            ssidDelegateProperties.fontFamily = arabicFamily;
            textMetrics.font.pointSize        = size * 1.2;
            textMetrics.font.family           = arabicFamily;
            psw_label.font.pointSize          = size * 1.2;
            psw_label.font.family             = arabicFamily;
            psw.font.pointSize                = size * 1.2;
            psw.font.family                   = arabicFamily;
            submit.font.pointSize             = size * 1.2;
            submit.font.family                = arabicFamily;
            cancel.font.pointSize             = size * 1.2;
            cancel.font.family                = arabicFamily;
            return;
        }

        ssidDelegateProperties.fontSize   = size;
        ssidDelegateProperties.fontFamily = latinFamily;
        textMetrics.font.pointSize        = size;
        textMetrics.font.family           = latinFamily;
        psw_label.font.pointSize          = size;
        psw_label.font.family             = latinFamily;
        psw.font.pointSize                = size;
        psw.font.family                   = latinFamily;
        wifiSubmit.font.pointSize         = size;
        wifiSubmit.font.family            = latinFamily;
        wifiCancel.font.pointSize         = size;
        wifiCancel.font.family            = latinFamily;
    }

    function setList(list)
    {
        var textHeight = 0;

        ssidsModel.clear();

        if (list.length < 1) {
            return;
        }

        for (var i = 0; i < list.length; i++) {
            ssidsModel.append({ displayStr: list[i] });

            textMetrics.text = list[i];
            if (textMetrics.height > textHeight) {
                textHeight = textMetrics.height;
            }
        }

        ssidList.currentIndex = -1;
        if (textHeight > 0) {
            ssidDelegateProperties.height = textHeight;
        }
    }

    function setWifiBusy(busy)
    {
        wifiBusy.visible = busy;
    }


    QtObject {
        id: ssidDelegateProperties

        property int    fontSize  : 8
        property string fontFamily: "Sans"
        property int    height    : 16
    }

    ListModel {
        id: ssidsModel
    }

    TextMetrics {
        id: textMetrics
    }


    Flickable {
        x: 0
        y: 0
        width: parent.width
        height: parent.height
        leftMargin: 3
        rightMargin: 3
        topMargin: 3
        clip: true
        contentHeight: ssidList.height

        ScrollBar.vertical: ScrollBar { }

        ListView {
            id: ssidList
            height: (ssidDelegateProperties.height + 6) * ssidsModel.count
            width: parent.parent.width
            clip: true
            delegate: Component {
                id: ssidDelegate

                MouseArea {
                    height: ssidDelegateProperties.height + 6
                    width: ssidList.width

                    acceptedButtons: Qt.LeftButton
                    onClicked: ssidList.currentIndex = index;

                    Label {
                        font.family: ssidDelegateProperties.fontSize
                        font.pointSize: ssidDelegateProperties.fontSize
                        height: ssidDelegateProperties.height + 6
                        padding: 3
                        text: displayStr
                        width: ssidList.width
                    }
                }
            }
            highlight: Rectangle {
                radius: 3
                color: "LightSteelBlue";
            }
            model: ssidsModel
        }
    }

    Row {
        anchors.bottom: wifiSubmit.top
        anchors.left: parent.left
        anchors.right: parent.right
        anchors.margins: 3
        spacing: 3

        Label {
            id: psw_label
            anchors.verticalCenter: psw.verticalCenter
            text: qsTr("Password")
            wrapMode: Label.WrapAtWordBoundaryOrAnywhere
        }
        TextField {
            id: psw
            enabled: ssidList.currentIndex >= 0
        }
    }

    Label {
        id: wifiError
        anchors.left: parent.left
        anchors.right: wifiCancel.left
        anchors.verticalCenter: wifiCancel.verticalCenter
        anchors.leftMargin: 7
        anchors.rightMargin: 7
        font.bold: true
        font.italic: true
        color: "#7A5544"
    }

    Button {
        id: wifiSubmit
        anchors.bottom: parent.bottom
        anchors.bottomMargin: 3
        anchors.right: parent.right
        anchors.rightMargin: 3
        enabled: ssidList.currentIndex >= 0
        text: qsTr("OK")
        onClicked: {
            wifiSelectPressed(ssidsModel.get(ssidList.currentIndex).displayStr, psw.text);
        }
    }
    Button {
        id: wifiCancel
        anchors.bottom: parent.bottom
        anchors.bottomMargin: 3
        anchors.right: wifiSubmit.left
        anchors.rightMargin: 3
        text: qsTr("Cancel")
        onClicked: wifiCancelPressed();
    }

    BusyIndicator {
        id: wifiBusy
        anchors.verticalCenter: parent.verticalCenter
        anchors.horizontalCenter: parent.horizontalCenter
        width: parent.width > 100 ? 100 : parent.width - 20
        height: width
        visible: false
    }
}
