import QtQuick 2.12
import QtQuick.Window 2.12
import QtQuick.Controls 2.3
import QtQuick.VirtualKeyboard.Settings 2.2


Window {
    id: mainwindow
    visible: true
    title: "Efendi Adhan Clock"

    signal buttonPressed(int role);
    signal checkForUpdatesPressed();
    signal quranAudioTogglePressed(int role, int surah, int ayah);
    signal quranAyahChanged(int surah, int ayah);
    signal quranPressed();
    signal shutDownPressed();
    signal settingsCountrySelected(string geonameId, string countryCode);
    signal settingsLocationSearchPressed(string countryGeonameId, string countryCode, string regionGeonameId, string regionCode, string searchText);
    signal settingsUpdated(string settingsJSON);
    signal settingsSoundTestPressed(int event, int soundId);
    signal wifiSelected(string ssid, string psw);
    signal wifiCanceled();


    function hideBkg()
    {
        internal.bkgVisible = false;
        background.visible = false;
        mainScreen.hideBackgrounds();
    }

    function setBkg(image)
    {
        if (!background.visible) {
            internal.bkgVisible = true;
            background.visible = true;
            mainScreen.showBackgrounds();
        }
        background.swapImage(image);
    }

    function setCurrentPrayer(prayerName)
    {
        mainScreen.setCurrentPrayer(prayerName);
    }

    function setDate(gregorianDate, hijriDate, shamsiDate)
    {
        mainScreen.setDate(gregorianDate, hijriDate, shamsiDate);
    }

    function setErrorStr(errorStr, isTemporary)
    {
        errorStrTemporaryTimer.stop();

        mainScreen.setErrorStr(errorStr);
        settingsScreen.setErrorStr(errorStr);
        quranScreen.setErrorStr(errorStr);
        wifiScreen.setErrorStr(errorStr);

        if (!isTemporary) {
            internal.lastPermanentErrorStr = errorStr;
            return;
        }

        errorStrTemporaryTimer.start();
    }

    function setFont(latinFamily, arabicFamily, size, prayersArabic, settingsArabic)
    {
        mainScreen.setFont(latinFamily, arabicFamily, size, prayersArabic);
        settingsScreen.setFont(latinFamily, arabicFamily, size, prayersArabic, settingsArabic);
        quranScreen.setFont(latinFamily, arabicFamily, size, settingsArabic);
        wifiScreen.setFont(latinFamily, arabicFamily, size, settingsArabic);
    }

    function setLocation(cityName, regionName, countryName, timezoneName)
    {
        mainScreen.setLocation(cityName, regionName, countryName, timezoneName);
    }

    function setPrayer(number, name, timeString, highlightPercent)
    {
        mainScreen.setPrayer(number, name, timeString, highlightPercent);
    }

    function setScreen(screen)
    {
        if (screen === 0) {
            settingsScreen.visible = false;
            quranScreen.visible    = false;
            wifiScreen.visible     = false;
            mainScreen.visible     = true;
            background.visible     = internal.bkgVisible;
        }
        if (screen === 1) {
            mainScreen.visible     = false;
            quranScreen.visible    = false;
            wifiScreen.visible     = false;
            settingsScreen.visible = true;
            background.visible     = false;
        }
        if (screen === 2) {
            settingsScreen.visible = false;
            mainScreen.visible     = false;
            wifiScreen.visible     = false;
            quranScreen.visible    = true;
            background.visible     = internal.bkgVisible;
        }
        if (screen === 3) {
            mainScreen.visible     = false;
            quranScreen.visible    = false;
            settingsScreen.visible = false;
            wifiScreen.visible     = true;
            background.visible     = false;
        }
    }

    function setTime(westernTime, easternTime, farsiTime)
    {
        mainScreen.setTime(westernTime, easternTime, farsiTime);
    }

    function setUIButtonRole(role)
    {
        mainScreen.setUIButtonRole(role);
    }



    function settingsBkgDirCtrlVisible(isVisible)
    {
        settingsScreen.setBkgDirCtrlVisible(isVisible);
    }

    function settingsCheckForUpdatesVisible(isVisible)
    {
        settingsScreen.setCheckForUpdatesButtonVisible(isVisible);
    }

    function settingsCountries(countriesMap)
    {
        settingsScreen.setCountries(countriesMap);
    }

    function settingsInit(optionsMap)
    {
        settingsScreen.settingsInit(optionsMap);
    }

    function settingsLocationBusy(busy)
    {
        settingsScreen.setLocationBusy(busy);
    }

    function settingsRegions(regions)
    {
        settingsScreen.setRegions(regions);
    }

    function settingsSearchResults(results)
    {
        settingsScreen.setSearchResults(results);
    }


    function quranAudioToggleRole(isStop)
    {
        quranScreen.audioToggleRole(isStop)
    }

    function quranAyah(text, fontFamily)
    {
        quranScreen.setAyah(text, fontFamily);
    }

    function quranBusy(busy)
    {
        quranScreen.setBusy(busy);
    }

    function quranButtonVisible(visible)
    {
        mainScreen.setQuranButtonVisible(visible);
    }

    function quranNextAyah(sendAudioToggle)
    {
        quranScreen.nextAyah(sendAudioToggle);
    }

    function quranSuwar(suwar, selected, ayahSelected)
    {
        quranScreen.setSuwar(suwar, selected, ayahSelected);
    }


    function wifiSetList(list)
    {
        wifiScreen.setList(list);
    }

    function wifiBusy(busy)
    {
        wifiScreen.setWifiBusy(busy);
    }


    QtObject {
        id: internal

        property bool   bkgVisible: false
        property string lastPermanentErrorStr: ""
    }

    Timer {
        id: errorStrTemporaryTimer
        interval: 5000

        onTriggered: {
            mainScreen.setErrorStr(internal.lastPermanentErrorStr);
            settingsScreen.setErrorStr(internal.lastPermanentErrorStr);
            quranScreen.setErrorStr(internal.lastPermanentErrorStr);
            errorStrTemporaryTimer.stop();
        }
    }


    ImgSwp {
        id: background
        anchors.fill: parent
        visible: false
        swapDuration: 2500
    }

    MainScreen {
        id: mainScreen
        anchors.fill: parent
        visible: false

        onUiButtonPressed: buttonPressed(role);
        onQuranButtonPressed: quranPressed();
    }

    SettingsScreen {
        id: settingsScreen
        anchors.fill: parent
        visible: false

        Component.onCompleted: {
            VirtualKeyboardSettings.fullScreenMode = true;
        }

        onCheckForUpdatesButtonPressed: checkForUpdatesPressed();
        onCountrySelected: settingsCountrySelected(geonameId, countryCode)
        onCustomLocationSearchPressed: settingsLocationSearchPressed(countryGeonameId, countryCode, regionGeonameId, regionCode, searchText)
        onSettingsSubmitPressed: {
            settingsUpdated(settingsJSON);
            setScreen(0);
        }
        onSettingsCancelPressed: setScreen(0)
        onShutDownButtonPressed: shutDownPressed();
        onSoundTestPressed: settingsSoundTestPressed(event, soundId);
    }

    QuranScreen {
        id: quranScreen
        anchors.fill: parent
        visible: false

        onClosePressed: setScreen(0)
        onAyahChanged: quranAyahChanged(surah, ayah)
        onAudioTogglePressed: quranAudioTogglePressed(role, surah, ayah);
    }

    WiFi {
        id: wifiScreen
        anchors.fill: parent
        visible: false

        onWifiSelectPressed: {
            wifiSelected(ssid, psw);
            setScreen(0);
        }
        onWifiCancelPressed: {
            wifiCanceled();
            setScreen(0);
        }
    }
}
