#include "prayertimes.h"

PrayerTimes::PrayerTimes()
{
    valid = false;
}

CALCULATION_METHODS PrayerTimes::autoDetectMethod(QString countryCode)
{
    // Egypt
    if (countryCode.compare("EG") == 0) {
        return METHOD_EGYPT;
    }

    // Canada, Unites States Of America
    if ((countryCode.compare("CA") == 0) || (countryCode.compare("US") == 0)) {
        return METHOD_NORTH_AMERICA;
    }

    // Afghanistan, Bangladesh, India, Pakistan
    if (
        (countryCode.compare("AF") == 0) || (countryCode.compare("BD") == 0) ||
        (countryCode.compare("IN") == 0) || (countryCode.compare("PK") == 0)
       ) {
        return METHOD_PAKISTAN;
    }

    // United Arab Emirates, Bahrain, Kuwait, Oman, Qatar, Saudi Arabia, Yemen
    if (
        (countryCode.compare("AE") == 0) || (countryCode.compare("BH") == 0) ||
        (countryCode.compare("KW") == 0) || (countryCode.compare("OM") == 0) ||
        (countryCode.compare("QA") == 0) || (countryCode.compare("SA") == 0) ||
        (countryCode.compare("YE") == 0)
    ) {
        return METHOD_SAUDI;
    }

    // Iran
    if (countryCode.compare("IR") == 0) {
        return METHOD_IRAN;
    }

    return METHOD_REST_OF_THE_WORLD;
}


CALCULATION_SCHOOLS PrayerTimes::autoDetectSchool(QString countryCode)
{
    // Afghanistan, Albania, Bosnia And Herzegovina, Bangladesh, Bulgaria, Bhutan, China, Cyprus, Egypt,
    // Greece, Israel, India, Iraq, Jordan, Kazakhstan, Kyrgyzstan, Lebanon, Montenegro, Macedonia, Nepal,
    // Tajikistan, Turkmenistan, Turkey, Palestine, Pakistan, Serbia, Russia, Syria, Uzbekistan, Kosovo
    if (
        (countryCode.compare("AF") == 0) || (countryCode.compare("AL") == 0) || (countryCode.compare("BA") == 0) ||
        (countryCode.compare("BD") == 0) || (countryCode.compare("BG") == 0) || (countryCode.compare("BT") == 0) ||
        (countryCode.compare("CN") == 0) || (countryCode.compare("CY") == 0) || (countryCode.compare("EG") == 0) ||
        (countryCode.compare("GR") == 0) || (countryCode.compare("IL") == 0) || (countryCode.compare("IN") == 0) ||
        (countryCode.compare("IQ") == 0) || (countryCode.compare("JO") == 0) || (countryCode.compare("KZ") == 0) ||
        (countryCode.compare("KG") == 0) || (countryCode.compare("LB") == 0) || (countryCode.compare("ME") == 0) ||
        (countryCode.compare("MK") == 0) || (countryCode.compare("NP") == 0) || (countryCode.compare("TJ") == 0) ||
        (countryCode.compare("TM") == 0) || (countryCode.compare("TR") == 0) || (countryCode.compare("PS") == 0) ||
        (countryCode.compare("PK") == 0) || (countryCode.compare("RS") == 0) || (countryCode.compare("RU") == 0) ||
        (countryCode.compare("SY") == 0) || (countryCode.compare("UZ") == 0) || (countryCode.compare("XK") == 0)
    ) {
        return SCHOOL_HANAFI;
    }

    return SCHOOL_HANBALI_MALIKI_SHAFI;
}


void PrayerTimes::calculate()
{
    valid = false;

    QSettings settings;

    QString group = settings.childGroups().contains("locationCustom") ? "locationCustom" : "locationAuto";

    QString timeZoneId = settings.value(QString("%1/timeZone").arg(group), "").toString();
    if (timeZoneId.isEmpty()) {
        errorMsg = "Time zone unknown";
        return;
    }
    QTimeZone timeZone(timeZoneId.toUtf8());

    bool   OK       = false;
    double latitude = settings.value(QString("%1/latitude").arg(group), 0).toDouble(&OK);
    if (!OK) {
        errorMsg = "Latitude unknown";
        return;
    }
    double longitude = settings.value(QString("%1/longitude").arg(group), 0).toDouble(&OK);
    if (!OK) {
        errorMsg = "Longitude unknown";
        return;
    }
    if ((latitude < -90) || (latitude > 90) || (longitude < -180) || (longitude > 180)) {
        errorMsg = "Invalid coordinates";
        return;
    }

    QString countryCode = settings.value(QString("%1/countryCode").arg(group), "").toString();
    if (countryCode.isEmpty()) {
        errorMsg = "Country unknown";
        return;
    }

    CALCULATION_METHODS           method          = static_cast<CALCULATION_METHODS>(settings.value("prefs/prayer_times_method", DEFAULT_PRAYER_METHOD).toInt());
    CALCULATION_SCHOOLS           school          = static_cast<CALCULATION_SCHOOLS>(settings.value("prefs/prayer_times_school", DEFAULT_PRAYER_SCHOOL).toInt());
    CALCULATION_HIGH_LATITUDES    highLatitude    = static_cast<CALCULATION_HIGH_LATITUDES>(settings.value("prefs/prayer_times_high_latitude", DEFAULT_HIGH_LATITUDE).toInt());
    CALCULATION_EXTREME_LATITUDES extremeLatitude = static_cast<CALCULATION_EXTREME_LATITUDES>(settings.value("prefs/prayer_times_extreme_latitude", DEFAULT_EXTREME_LATITUDE).toInt());
    if (method == METHOD_AUTODETECT) {
        method = autoDetectMethod(countryCode);
    }
    if (school == SCHOOL_AUTODETECT) {
        school = autoDetectSchool(countryCode);
    }
    if (highLatitude == HIGH_LATITUDE_DEFAULT) {
        highLatitude = HIGH_LATITUDE_SEVENTH_OF_NIGHT;
    }
    if (extremeLatitude == EXTREME_LATITUDE_DEFAULT) {
        extremeLatitude = EXTREME_LATITUDE_NEAREST_LATITUDE;
    }

    double fajrAngle    = getFajrAngle(method);
    double ishaAngle    = getIshaAngle(method);
    double shadowLength = getShadow(school);
    if (fajrAngle == 0) {
        errorMsg = "Invalid method";
        return;
    }

    QDateTime now = QDateTime::currentDateTimeUtc().toTimeZone(timeZone);

    int    dayOfYear = now.date().dayOfYear();
    int    offset    = timeZone.offsetFromUtc(now);
    double tzDegrees = static_cast<double>(offset) / 3600 * 15;

    double declanationOfSun = calculateDeclinationOfSun(dayOfYear);
    double equationOfTime   = calculateEquationOfTime(dayOfYear);
    double midday           = 12 + ((tzDegrees - longitude) / 15) - (equationOfTime / 60);

    double fajrFractional    = regularPrayerTime(midday, fajrAngle * -1, latitude, declanationOfSun, 0);
    double shurooqFractional = regularPrayerTime(midday, SUN_NATURAL_DEGREE * -1, latitude, declanationOfSun, 0);
    double dhuhrFractional   = midday + 0.0167;                                                                     // plus one minute because it should not be exactly at noon
    double asrFractional     = afternoonPrayerTime(midday, shadowLength, latitude, declanationOfSun);
    double maghribFractional = regularPrayerTime(midday, SUN_NATURAL_DEGREE, latitude, declanationOfSun, 0);
    double ishaFractional;
    if (ishaAngle == 0) {
        ishaFractional = regularPrayerTime(midday, SUN_NATURAL_DEGREE, latitude, declanationOfSun, 90);
    }
    else {
        ishaFractional = regularPrayerTime(midday, ishaAngle, latitude, declanationOfSun, 0);
    }

    double fastingHours = maghribFractional - fajrFractional;
    if ((highLatitude != HIGH_LATITUDE_NONE) && ((fastingHours < FASTING_MIN_HOURS) || (fastingHours > FASTING_MAX_HOURS))) {
        HighLatitudeCorrection correctedFajrIsha = highLatitudeCorrection(highLatitude, fajrAngle, ishaAngle, shurooqFractional, maghribFractional);
        fajrFractional = correctedFajrIsha.fajrCorrected;
        ishaFractional = correctedFajrIsha.ishaCorrected;
        fastingHours   = maghribFractional - fajrFractional;
    }
    if ((extremeLatitude != EXTREME_LATITUDE_NONE) && ((fastingHours < FASTING_MIN_HOURS) || (fastingHours > FASTING_MAX_HOURS))) {
        if (extremeLatitude == EXTREME_LATITUDE_NEAREST_LATITUDE) {
            double latitudeVar = latitude < 0 ? qCeil(latitude) : qFloor(latitude);
            while (latitudeVar != 0) {
                HighLatitudeCorrection correctedFajrIsha = {
                    regularPrayerTime(midday, EXTREME_LATITUDE_FAJR_ISHA * -1, latitudeVar, declanationOfSun, 0),
                    regularPrayerTime(midday, EXTREME_LATITUDE_FAJR_ISHA, latitudeVar, declanationOfSun, 0),
                };

                double lShurooqFractional = regularPrayerTime(midday, SUN_NATURAL_DEGREE * -1, latitudeVar, declanationOfSun, 0);
                double lMaghribFractional = regularPrayerTime(midday, SUN_NATURAL_DEGREE, latitudeVar, declanationOfSun, 0);

                if (highLatitude != HIGH_LATITUDE_NONE) {
                    correctedFajrIsha = highLatitudeCorrection(highLatitude, EXTREME_LATITUDE_FAJR_ISHA * -1, EXTREME_LATITUDE_FAJR_ISHA, lShurooqFractional, lMaghribFractional);
                }

                double newFastingHours = lMaghribFractional - correctedFajrIsha.fajrCorrected;
                if ((newFastingHours >= FASTING_MIN_HOURS) && (newFastingHours <= FASTING_MAX_HOURS)) {
                    fajrFractional    = correctedFajrIsha.fajrCorrected;
                    shurooqFractional = lShurooqFractional;
                    asrFractional     = afternoonPrayerTime(midday, shadowLength, latitudeVar, declanationOfSun);
                    maghribFractional = lMaghribFractional;
                    ishaFractional    = correctedFajrIsha.ishaCorrected;
                    break;
                }

                latitudeVar = latitudeVar < 0 ? latitudeVar + 1 : latitudeVar - 1;
            }
        }
        else if (extremeLatitude == EXTREME_LATITUDE_NEAREST_DAY) {
            QDateTime back(now);
            QDateTime forward(now);

            int i = 0;
            while (i < 183) {
                back    = back.addDays(-1);
                forward = forward.addDays(1);

                {
                    int    bDayOfYear = back.date().dayOfYear();
                    int    bOffset    = timeZone.offsetFromUtc(back);
                    double bTZDegrees = (double) bOffset / 3600 * 15;

                    double bDeclanationOfSun = calculateDeclinationOfSun(bDayOfYear);
                    double bEquationOfTime   = calculateEquationOfTime(bDayOfYear);
                    double bMidday           = 12 + ((bTZDegrees - longitude) / 15) - (bEquationOfTime / 60);

                    HighLatitudeCorrection correctedFajrIsha = {
                        regularPrayerTime(bMidday, EXTREME_LATITUDE_FAJR_ISHA * -1, latitude, bDeclanationOfSun, 0),
                        regularPrayerTime(bMidday, EXTREME_LATITUDE_FAJR_ISHA, latitude, bDeclanationOfSun, 0),
                    };

                    double bShurooqFractional = regularPrayerTime(bMidday, SUN_NATURAL_DEGREE * -1, latitude, bDeclanationOfSun, 0);
                    double bMaghribFractional = regularPrayerTime(bMidday, SUN_NATURAL_DEGREE, latitude, bDeclanationOfSun, 0);

                    if (highLatitude != HIGH_LATITUDE_NONE) {
                        correctedFajrIsha = highLatitudeCorrection(highLatitude, EXTREME_LATITUDE_FAJR_ISHA, EXTREME_LATITUDE_FAJR_ISHA, bShurooqFractional, bMaghribFractional);
                    }

                    double newFastingHours = bMaghribFractional - correctedFajrIsha.fajrCorrected;
                    if ((newFastingHours >= FASTING_MIN_HOURS) && (newFastingHours <= FASTING_MAX_HOURS)) {
                        fajrFractional    = correctedFajrIsha.fajrCorrected;
                        shurooqFractional = bShurooqFractional;
                        dhuhrFractional   = bMidday + 0.0167;
                        asrFractional     = afternoonPrayerTime(bMidday, shadowLength, latitude, bDeclanationOfSun);
                        maghribFractional = bMaghribFractional;
                        ishaFractional    = correctedFajrIsha.ishaCorrected;
                        break;
                    }
                }
                {
                    int    fDayOfYear = forward.date().dayOfYear();
                    int    fOffset    = timeZone.offsetFromUtc(forward);
                    double fTZDegrees = (double) fOffset / 3600 * 15;

                    double fDeclanationOfSun = calculateDeclinationOfSun(fDayOfYear);
                    double fEquationOfTime   = calculateEquationOfTime(fDayOfYear);
                    double fMidday           = 12 + ((fTZDegrees - longitude) / 15) - (fEquationOfTime / 60);

                    HighLatitudeCorrection correctedFajrIsha = {
                        regularPrayerTime(fMidday, EXTREME_LATITUDE_FAJR_ISHA * -1, latitude, fDeclanationOfSun, 0),
                        regularPrayerTime(fMidday, EXTREME_LATITUDE_FAJR_ISHA, latitude, fDeclanationOfSun, 0),
                    };

                    double fShurooqFractional = regularPrayerTime(fMidday, SUN_NATURAL_DEGREE * -1, latitude, fDeclanationOfSun, 0);
                    double fMaghribFractional = regularPrayerTime(fMidday, SUN_NATURAL_DEGREE, latitude, fDeclanationOfSun, 0);

                    if (highLatitude != HIGH_LATITUDE_NONE) {
                        correctedFajrIsha = highLatitudeCorrection(highLatitude, EXTREME_LATITUDE_FAJR_ISHA, EXTREME_LATITUDE_FAJR_ISHA, fShurooqFractional, fMaghribFractional);
                    }

                    double newFastingHours = fMaghribFractional - correctedFajrIsha.fajrCorrected;
                    if ((newFastingHours >= FASTING_MIN_HOURS) && (newFastingHours <= FASTING_MAX_HOURS)) {
                        fajrFractional    = correctedFajrIsha.fajrCorrected;
                        shurooqFractional = fShurooqFractional;
                        dhuhrFractional   = fMidday + 0.0167;
                        asrFractional     = afternoonPrayerTime(fMidday, shadowLength, latitude, fDeclanationOfSun);
                        maghribFractional = fMaghribFractional;
                        ishaFractional    = correctedFajrIsha.ishaCorrected;
                        break;
                    }
                }

                i++;
            }
        }
    }

    fajr    = QTime::fromMSecsSinceStartOfDay(fajrFractional * 60 * 60 * 1000);
    shurooq = QTime::fromMSecsSinceStartOfDay(shurooqFractional * 60 * 60 * 1000);
    dhuhr   = QTime::fromMSecsSinceStartOfDay(dhuhrFractional * 60 * 60 * 1000);
    asr     = QTime::fromMSecsSinceStartOfDay(asrFractional * 60 * 60 * 1000);
    maghrib = QTime::fromMSecsSinceStartOfDay(maghribFractional * 60 * 60 * 1000);
    isha    = QTime::fromMSecsSinceStartOfDay(ishaFractional * 60 * 60 * 1000);

    fajr.setHMS(fajr.hour(), fajr.minute(), 0);
    shurooq.setHMS(shurooq.hour(), shurooq.minute(), 0);
    dhuhr.setHMS(dhuhr.hour(), dhuhr.minute(), 0);
    asr.setHMS(asr.hour(), asr.minute(), 0);
    maghrib.setHMS(maghrib.hour(), maghrib.minute(), 0);
    isha.setHMS(isha.hour(), isha.minute(), 0);

    valid = true;
}


double PrayerTimes::getFajrAngle(int method)
{
    switch (method) {
        case METHOD_EGYPT:
            return METHOD_EGYPT_FAJR;
        case METHOD_IRAN:
            return METHOD_IRAN_FAJR;
        case METHOD_NORTH_AMERICA:
            return METHOD_NORTHAMERICA_FAJR;
        case METHOD_PAKISTAN:
            return METHOD_PAKISTAN_FAJR;
        case METHOD_SAUDI:
            return METHOD_SAUDI_FAJR;
    }
    return METHOD_EUANDWORLD_FAJR;
}


double PrayerTimes::getIshaAngle(int method)
{
    switch (method) {
        case METHOD_EGYPT:
            return METHOD_EGYPT_ISHA;
        case METHOD_IRAN:
            return METHOD_IRAN_ISHA;
        case METHOD_NORTH_AMERICA:
            return METHOD_NORTHAMERICA_ISHA;
        case METHOD_PAKISTAN:
            return METHOD_PAKISTAN_ISHA;
        case METHOD_SAUDI:
            return METHOD_SAUDI_ISHA;
    }
    return METHOD_EUANDWORLD_ISHA;
}


double PrayerTimes::getShadow(int school)
{
    if (school == SCHOOL_HANAFI) {
        return SCHOOL_HANAFI_SHADOW;
    }
    return SCHOOL_HANBALI_MALIKI_SHAFI_SHADOW;
}


PrayerTimes::HighLatitudeCorrection PrayerTimes::highLatitudeCorrection(int highLatitude, double fajrAngle, double ishaAngle, double shurooqFractional, double maghribFractional)
{
    HighLatitudeCorrection returnValue;

    double nightHours = 24.0 - (maghribFractional - shurooqFractional);

    if (highLatitude == HIGH_LATITUDE_MIDDLE_OF_NIGHT) {
        returnValue.fajrCorrected = shurooqFractional - nightHours * .25;
        returnValue.ishaCorrected = maghribFractional + nightHours * .25;
    }
    else if (highLatitude == HIGH_LATITUDE_SEVENTH_OF_NIGHT) {
        returnValue.fajrCorrected = shurooqFractional - nightHours / 7;
        returnValue.ishaCorrected = maghribFractional + nightHours / 7;
    }
    else if (highLatitude == HIGH_LATITUDE_FRACTION_OF_NIGHT) {
        returnValue.fajrCorrected = shurooqFractional - nightHours * (fajrAngle / 60);
        returnValue.ishaCorrected = maghribFractional + nightHours * (ishaAngle == 0 ? 0.3 : ishaAngle / 60);
    }
    else {
        returnValue.fajrCorrected = 0.0167;
        returnValue.ishaCorrected = 23.9833;
    }

    return returnValue;
}


double PrayerTimes::calculateDeclinationOfSun(int dayOfYear)
{
    double temp = ((2 * M_PI) / 365) * dayOfYear;
    return (180 / M_PI) * (0.006918 - 0.399912 * cos(temp) + 0.070257 * sin(temp) - 0.006758 * cos(2 * temp) + 0.000907 * sin(2 * temp) - 0.002697 * cos(3 * temp) + 0.00148 * sin(3 * temp));
}


double PrayerTimes::calculateEquationOfTime(int dayOfYear)
{
    double temp = (2 * M_PI) * (dayOfYear - 80) / 364;
    return 9.87 * sin(2 * temp) - 7.53 * cos(temp) - 1.5 * sin(temp);
}


double PrayerTimes::rad2deg(double what)
{
    return (180 * what / M_PI);
}


double PrayerTimes::deg2rad(double what)
{
    return (M_PI * what / 180);
}


double PrayerTimes::timeBetweenNoonAndSunAtAngle(double angle, double latitude, double declanation)
{
    return ((double) 1 / (double) 15) * (rad2deg(acos(((-1 * sin(deg2rad(angle))) - (sin(deg2rad(latitude)) * sin(deg2rad(declanation)))) / (cos(deg2rad(latitude)) * cos(deg2rad(declanation))))));
}


double PrayerTimes::timeBetweenNoonAndShadowLenght(double shadowLength, double latitude, double declanation)
{
    return ((double) 1 / (double) 15) * (rad2deg(acos((sin((M_PI / 2) - atan(shadowLength + tan(abs(deg2rad(latitude) - deg2rad(declanation))))) - (sin(deg2rad(latitude)) * sin(deg2rad(declanation)))) / (cos(deg2rad(latitude)) * cos(deg2rad(declanation))))));
}


double PrayerTimes::regularPrayerTime(double midday, double angle, double latitude, double declanationOfSun, int offsetMins)
{
    double temp;

    if (angle < 0) {
        angle *= -1;
        temp = midday - timeBetweenNoonAndSunAtAngle(angle, latitude, declanationOfSun);
    }
    else {
        temp = midday + timeBetweenNoonAndSunAtAngle(angle, latitude, declanationOfSun);
    }

    temp += static_cast<double>(offsetMins) / 60;

    return temp;
}


double PrayerTimes::afternoonPrayerTime(double midday, double shadowLength, double latitude, double declanationOfSun)
{
    double temp = midday + timeBetweenNoonAndShadowLenght(shadowLength, latitude, declanationOfSun);
    return temp;
}


bool PrayerTimes::isValid()
{
    return valid;
}


QString PrayerTimes::lastError()
{
    return errorMsg;
}


QTime PrayerTimes::getFajr()
{
    return fajr;
}


QTime PrayerTimes::getShurooq()
{
    return shurooq;
}


QTime PrayerTimes::getDhuhr()
{
    return dhuhr;
}


QTime PrayerTimes::getAsr()
{
    return asr;
}


QTime PrayerTimes::getMaghrib()
{
    return maghrib;
}


QTime PrayerTimes::getIsha()
{
    return isha;
}


QTime PrayerTimes::getPrayer(PRAYER_EVENTS prayerEvent)
{
    switch (prayerEvent) {
        case FAJR:
            return fajr;
        case SHUROOQ:
            return shurooq;
        case DHUHR:
            return dhuhr;
        case ASR:
            return asr;
        case MAGHRIB:
            return maghrib;
        case ISHA:
            return isha;
    }
    return QTime();
}


PrayerTimes::PrayerEvent PrayerTimes::getPrevious()
{
    PrayerEvent returnValue = { NONE, QTime(), 0, 0 };

    if (!valid) {
        return returnValue;
    }

    QSettings settings;

    QString timeZoneId = settings.value(QString("%1/timeZone").arg(settings.childGroups().contains("locationCustom") ? "locationCustom" : "locationAuto"), "").toString();
    if (timeZoneId.isEmpty()) {
        return returnValue;
    }
    QTime now = QDateTime::currentDateTimeUtc().toTimeZone(QTimeZone(timeZoneId.toUtf8())).time();

    int highlightSecs = settings.value("prefs/last_prayer_highlight_minutes",  DEFAULT_HIGHLIGHT_LAST).toInt() * 60;

    foreach (PRAYER_EVENTS event, QList<PRAYER_EVENTS>({ ISHA, MAGHRIB, ASR, DHUHR, SHUROOQ, FAJR })) {
        int realHighlightSecs = event == SHUROOQ ? SHUROOQ_MINUTES * 60 : highlightSecs;
        int seconds            = eventsToTime.value(event)->secsTo(now);

        if (seconds < 0) {
            continue;
        }

        returnValue.prayerEvent = event;
        returnValue.time        = QTime(eventsToTime.value(event)->hour(), eventsToTime.value(event)->minute());
        returnValue.seconds     = seconds;

        if (seconds <= realHighlightSecs) {
            returnValue.percent = (seconds == 0 ? 0.1 : static_cast<double>(seconds)) / realHighlightSecs;
        }

        break;
    }

    return returnValue;
}


PrayerTimes::PrayerEvent PrayerTimes::getNext()
{
    PrayerEvent returnValue = { NONE, QTime(), 0, 0 };

    if (!valid) {
        return returnValue;
    }

    QSettings settings;

    QString timeZoneId = settings.value(QString("%1/timeZone").arg(settings.childGroups().contains("locationCustom") ? "locationCustom" : "locationAuto"), "").toString();
    if (timeZoneId.isEmpty()) {
        return returnValue;
    }
    QTime now = QDateTime::currentDateTimeUtc().toTimeZone(QTimeZone(timeZoneId.toUtf8())).time();

    int   countdownSecs        = settings.value("prefs/next_prayer_highlight_minutes",  DEFAULT_HIGHLIGHT_NEXT).toInt() * 60;
    int   shurooqCountdownSecs = settings.value("prefs/next_shurooq_highlight_minutes", DEFAULT_HIGHLIGHT_SHUROOQ).toInt() * 60;

    foreach (PRAYER_EVENTS event, QList<PRAYER_EVENTS>({ FAJR, SHUROOQ, DHUHR, ASR, MAGHRIB, ISHA })) {
        int seconds = now.secsTo(*(eventsToTime.value(event)));

        if (seconds < 0) {
            continue;
        }

        returnValue.prayerEvent = event;
        returnValue.time        = QTime(eventsToTime.value(event)->hour(), eventsToTime.value(event)->minute());
        returnValue.seconds     = seconds;

        int secs = event == SHUROOQ ? shurooqCountdownSecs : countdownSecs;
        if (seconds <= secs) {
            returnValue.percent = static_cast<double>(seconds) / secs;
        }

        break;
    }

    return returnValue;
}
